# Finnish lexemes common.py
lexemes = {
    "app_title": "Notolog Editori",
    "app_title_with_sub": "{app_title} - {sub_title}",

    "tree_filter_accessible_desc": "Tiedostosuodattimen kenttä",

    "menu_action_copy_file_path": "Kopioi tiedostopolku",
    "menu_action_rename": "Nimeä uudelleen",
    "menu_action_delete": "Poista",
    "menu_action_delete_completely": "Poista kokonaan",
    "menu_action_restore": "Palauta",
    "menu_action_create_new_dir": "Luo uusi hakemisto",

    "dialog_file_rename_title": "Nimeä tiedosto uudelleen",
    "dialog_file_rename_field_label": "Anna uusi tiedostonimi",
    "dialog_file_rename_button_ok": "Nimeä uudelleen",
    "dialog_file_rename_warning_exists": "Samanniminen tiedosto on jo olemassa",

    "dialog_file_delete_title": "Poista tiedosto",
    "dialog_file_delete_text": "Poista tiedosto \"{file_name}\"?",
    "dialog_file_delete_completely_title": "Poista tiedosto kokonaan",
    "dialog_file_delete_completely_text": "Poista tiedosto \"{file_name}\" kokonaan?",
    "dialog_file_delete_error": "Tiedostoa ei voi poistaa, tapahtui virhe",
    "dialog_file_delete_error_not_found": "Tiedostoa ei löydy",

    "dialog_file_restore_title": "Palauta tiedosto",
    "dialog_file_restore_text": "Palauta tiedosto \"{file_name}\"?",
    "dialog_file_restore_error": "Tiedostoa ei voi palauttaa, tapahtui virhe",
    "dialog_file_restore_warning_exists": "Samanniminen tiedosto on jo olemassa",

    "dialog_create_new_dir_title": "Luo uusi hakemisto",
    "dialog_create_new_dir_label": "Uuden hakemiston nimi",
    "dialog_create_new_dir_input_placeholder_text": "Anna hakemiston nimi",
    "dialog_create_new_dir_button_ok": "Luo",
    "dialog_create_new_dir_button_cancel": "Peruuta",
    "dialog_create_new_dir_warning_empty_title": "Uuden hakemiston nimen virhe",
    "dialog_create_new_dir_warning_empty_text": "Hakemiston nimi ei voi olla tyhjä",
    "dialog_create_new_dir_warning_too_long_title": "Uuden hakemiston nimen virhe",
    "dialog_create_new_dir_warning_too_long_text": "Hakemiston nimi on liian pitkä; enintään "
                                                   "{symbols} merkkiä sallittu!",
    "dialog_create_new_dir_error_existed": "Hakemisto on jo olemassa",
    "dialog_create_new_dir_error": "Hakemistoa ei voi luoda. Varmista, että kohdehakemisto "
                                   "{base_dir} on kirjoitettavissa",

    "dialog_message_box_title": "Viesti",
    "dialog_message_box_button_ok": "Sulje",

    "action_new_file_first_line_template_text": "Uusi asiakirja",
    "action_open_file_dialog_caption": "Avaa tiedosto",
    "action_save_as_file_dialog_caption": "Tallenna tiedosto",

    "dialog_save_empty_file_title": "Tallenna tyhjä tiedosto",
    "dialog_save_empty_file_text": "Salli tiedoston tallentaminen ilman sisältöä?",

    "dialog_encrypt_file_title": "Salaa tiedosto",
    "dialog_encrypt_file_text": "Salaa tiedosto \"{file_name}\"?",
    "encrypt_file_warning_file_is_already_encrypted": "Tiedosto on jo salattu!",
    "dialog_encrypt_file_rewrite_existing_title": "Kirjoita olemassa oleva tiedosto uudelleen",
    "dialog_encrypt_file_rewrite_existing_text": "Kirjoita olemassa oleva tiedosto \"{file_path}\" uudelleen?",

    "dialog_decrypt_file_title": "Pura tiedoston salaus",
    "dialog_decrypt_file_text": "Pura tiedoston salaus \"{file_name}\"?",
    "decrypt_file_warning_file_is_not_encrypted": "Tiedosto ei ole salattu!",
    "dialog_decrypt_file_rewrite_existing_title": "Kirjoita olemassa oleva tiedosto uudelleen",
    "dialog_decrypt_file_rewrite_existing_text": "Kirjoita olemassa oleva tiedosto \"{file_path}\" uudelleen?",

    "dialog_encrypt_new_password_title": "Uusi salasana",
    "dialog_encrypt_new_password_label": "Salasana:",
    "dialog_encrypt_new_password_input_placeholder_text": "Anna uusi salasana",
    "dialog_encrypt_new_password_hint_label": "Vihje:",
    "dialog_encrypt_new_password_hint_label_description": "Vihje ei ole salattu ja sen voi lukea tiedostosta!"
                                                          "\nVältä ilmeisiä vihjeitä, joita voi helposti arvata, kuten "
                                                          "syntymäpäiviä."
                                                          "\nYritä käyttää viitettä, jota ei ole helppo yhdistää sinuun.",
    "dialog_encrypt_new_password_hint_input_placeholder_text": "Anna vihje (valinnainen)",
    "dialog_encrypt_new_password_button_ok": "OK",
    "dialog_encrypt_new_password_button_cancel": "Peruuta",
    "dialog_encrypt_new_password_warning_empty_title": "Varoitus",
    "dialog_encrypt_new_password_warning_empty_text": "Salasanakenttä ei voi olla tyhjä!",
    "dialog_encrypt_new_password_warning_too_long_title": "Varoitus",
    "dialog_encrypt_new_password_warning_too_long_text": "Vihjekenttä on liian pitkä, enintään {symbols} merkkiä!",

    "dialog_encrypt_password_title": "Anna salasana",
    "dialog_encrypt_password_label": "Salasana:",
    "dialog_encrypt_password_input_placeholder_text": "Anna salasana",
    "dialog_encrypt_password_hint_label": "Vihje:",
    "dialog_encrypt_password_button_ok": "OK",
    "dialog_encrypt_password_button_cancel": "Peruuta",

    "dialog_encrypt_password_reset_title": "Nollaa salauksen salasana",
    "dialog_encrypt_password_reset_text": "Haluatko varmasti nollata nykyisen salauksen salasanan?",
    "dialog_encrypt_password_reset_button_cancel": "Peruuta",
    "dialog_encrypt_password_reset_button_yes": "Kyllä",

    "dialog_open_link_title": "Linkki",
    "dialog_open_link_text": "Avaa linkki \"{url}\" selaimessa?",

    "dialog_reset_settings_title": "Nollaa asetukset?",
    "dialog_reset_settings_text":
        "Kaikki asetuksissa tallennetut tiedot poistetaan, ja sovellus käynnistetään uudelleen muutosten soveltamiseksi.",

    "dialog_exit_unsaved_title": "Vahvista Poistuminen",
    "dialog_exit_unsaved_text": "Avattua tiedostoa '{file_name}' ei voi tallentaa. Jatketaanko poistumista?",

    "message_app_config_file_access": "Käyttöoikeus evätty yrittäessä päästä sovelluksen konfiguraatiotiedostoon {file_path}. "
                                      "Aseta oikeat käyttöoikeudet varmistaaksesi asianmukaisen toiminnan.",

    "field_dir_path_dialog_caption": "Valitse Hakemisto",
    "field_file_path_dialog_caption": "Valitse Tiedosto",

    "load_file_encryption_password_mismatch": "Salaussalasana ei täsmää!",
    "load_file_encryption_password_incorrect": "Väärä salaussalasana!",
    "load_file_none_content_error": "Tiedostoa ei voi ladata.",

    "action_new_file_error_occurred": "Tiedostoa ei voida luoda; virhe tapahtui.\nTarkista tiedostojärjestelmän oikeudet.",
    "save_active_file_error_occurred": "Tiedostoa ei voida tallentaa; virhe tapahtui.",

    "expandable_block_default_title": "Lisätietoja...",
    "expandable_block_open_close_tags_mismatch_warning": "<details>-lohkon avaamis- ja sulkumerkkien epäjohdonmukaisuus",

    "dialog_color_picker_color_copied_to_the_clipboard": "Muotoiltu teksti on kopioitu leikepöydälle",

    "popup_about_title": "Sovelluksen tiedot",
    "popup_about_app_name_description": "Python Markdown -editori",

    "popup_about_version": "Versio",
    "popup_about_license": "Lisenssi",
    "popup_about_website": "Verkkosivusto",
    "popup_about_repository": "GitHub",
    "popup_about_pypi": "PyPi",
    "popup_about_date": "Päivämäärä",

    "update_helper_new_version_is_available": "Sovelluksen uusi versio {latest_version} on saatavilla",
    "update_helper_latest_version_installed": "Sovelluksen uusin versio on asennettu",

    "network_connection_error_empty": "Vastausinformaatiota ei voi hankkia",
    "network_connection_error_connection_or_dns":
        "Isäntää ei löydy. Internet-yhteydessä tai DNS:ssä voi olla ongelma.",
    "network_connection_error_connection_refused":
        "Yhteys hylätty. Palvelin voi olla alhaalla tai verkossa voi olla ongelmia.",
    "network_connection_error_connection_timed_out": "Yhteys katkesi aikakatkaisun vuoksi. Verkossa voi olla ongelmia.",
    "network_connection_error_connection_404_error":
        "Yhteys 404 virhe. Pyydettyä sivua tai resurssia ei löydy.",
    "network_connection_error_generic_with_status_code": "Pyyntö epäonnistui tilakoodilla: {status_code}",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: App themes manager.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

import os
import logging

from typing import Union

from PySide6.QtCore import QFile, QIODevice

from .app_config import AppConfig
from .helpers.file_helper import res_path
from .enums.themes import Themes


class Theme:

    ASSETS_DIR_NAME = "assets"

    _instance = None

    def __new__(cls, *args, **kwargs):
        if cls._instance is None or AppConfig().get_test_mode():  # No cache in test mode
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self, theme: str = None):
        # Load exact theme once
        if getattr(self, 'theme', None) and self.theme == theme:
            return

        self.logger = logging.getLogger('theme')

        # Custom Enum logic return name.lower() when cast to string
        # Same as: Themes.default().name.lower()
        self.default_theme = str(Themes.default())

        if not theme or theme in ('.', '..'):  # in case of file dependant
            theme = self.default_theme

        self.theme = theme

        # Can be seen in logs a few times as the already initialised object could be requested from anywhere.
        self.logger.info(f"Themes helper is activated and set to the theme: '{self.theme}'")

        self.colors = None
        self.css = None

        self.load_from_files()

    @staticmethod
    def get_assets_dir(join_parts: Union[str, list] = None) -> str:
        """
        Get the directory path with all assets files.
        @return: string with the themes directory path
        """
        join_parts = join_parts if isinstance(join_parts, list) else [join_parts] if isinstance(join_parts, str) else []
        return str(os.path.join(os.path.dirname(__file__), Theme.ASSETS_DIR_NAME, *join_parts))

    def get_themes_dir(self):
        """
        Get the directory path with themes files relative to this file.
        @return: string with the themes directory path
        """
        return self.get_assets_dir('themes')

    def get_theme_dir(self):
        """
        Get current theme directory.
        @return:
        """
        return os.path.join(self.get_themes_dir(), self.theme)

    def get_default_theme_dir(self):
        """
        Get current theme directory.
        @return:
        """
        return os.path.join(self.get_themes_dir(), self.default_theme)

    def load_from_files(self) -> None:
        # Init the vars
        self.colors = {}
        self.css = {}

        # Get current theme directory
        theme_dir = self.get_theme_dir()

        theme_files = []
        if os.path.isdir(theme_dir):
            theme_file_names = os.listdir(theme_dir)
            theme_file_paths = [os.path.join(theme_dir, _file_name) for _file_name in theme_file_names]
            for theme_file_path in theme_file_paths:
                if os.path.isfile(theme_file_path):
                    theme_files.append(theme_file_path)

        for file_path in theme_files:
            # Process css file
            if file_path.endswith('.css'):
                if self.theme not in self.css:
                    self.css[self.theme] = {}
                if os.path.isfile(file_path):
                    css_name = os.path.splitext(os.path.basename(file_path))[0]
                    self.css[self.theme].update({css_name: file_path})
                continue

            # Process colors file
            if not file_path.endswith('.ini'):
                continue

            if self.theme not in self.colors:
                self.colors[self.theme] = {}
            with open(file_path, 'r', encoding='utf-8') as file:
                for line in file:
                    # Remove leading/trailing whitespace
                    line = line.strip()
                    # Skip empty lines
                    if line:
                        parts = line.split('=')
                        if len(parts) == 2:
                            color_name = parts[0].strip()
                            color_value = parts[1].strip()
                            self.colors[self.theme][color_name] = color_value
                        else:
                            self.logger.debug(f"Ignoring not valid color line: {line}")

    def set_theme(self, theme: str) -> None:
        """
        Set theme and re-load all corresponded colors.
        @param theme: string key of the theme to load
        @return: None
        """
        self.theme = theme
        self.load_from_files()

    def get_colors(self, css_format: bool = False) -> Union[dict, None]:
        """
        Get the theme colors in a form of a dictionary.
        @param css_format: bool whether to convert to CSS format or not
        @return: dictionary with the theme colors
        """
        hex_colors_str = self.colors.get(self.theme, {}).items()
        if not hex_colors_str:
            return None
        hex_colors = {key: (int(value, 16) if value.startswith('0x') else value) for key, value in hex_colors_str}
        if css_format:
            # Convert each item value to CSS format (#RRGGBB)
            return dict({key: "#{:06x}".format(value).upper() for key, value in hex_colors})
        return dict(hex_colors)

    def get_css(self) -> Union[dict, None]:
        """
        Get the theme css in a form of a dictionary.
        @return: dictionary with the theme css
        """
        return self.css.get(self.theme, {})

    def load_css(self, path: str) -> Union[str, None]:
        """
        Load CSS file and return result as a string.
        QTextEdit implementation
        """
        file_path = QFile(res_path(path))
        if not file_path.open(QIODevice.OpenModeFlag.ReadOnly | QIODevice.OpenModeFlag.Text):
            return None
        # with open(file_path, 'r', encoding='utf-8') as file:
        #    css_content = file.read()
        return file_path.readAll().data().decode("utf-8")



# Greek lexemes statusbar.py
lexemes = {
    "statusbar_litter_bin_label": "Κάδος απορριμμάτων",
    "statusbar_litter_bin_accessible_name": "Εμφάνιση περιεχομένου κάδου απορριμμάτων",

    "statusbar_save_progress_label": b'\xf0\x9f\x92\xbe',  # Unicode for save icon
    "statusbar_encryption_symbol_encrypted_label": b'\xf0\x9f\x94\x92',  # Unicode for encrypted icon
    "statusbar_encryption_symbol_unencrypted_label": b'\xf0\x9f\x94\x93',  # Unicode for unencrypted icon

    "statusbar_mode_label_mode_view": "Λειτουργία προβολής",
    "statusbar_mode_label_mode_edit": "Λειτουργία επεξεργασίας",
    "statusbar_mode_label_mode_source": "Λειτουργία πηγής",

    "statusbar_source_label_source_markdown": "Markdown",
    "statusbar_source_label_source_html": "HTML",

    "statusbar_encryption_label": "{encryption} {icon}",
    "statusbar_encryption_label_encryption_plain": "Απλό",
    "statusbar_encryption_label_encryption_encrypted": "Κρυπτογραφημένο",

    "statusbar_cursor_label": "γραμμή: {line} → στήλη: {column}",
    "statusbar_cursor_label_selected": "γραμμή: {line} → στήλη: {column} → επιλ: {selected}",
    "statusbar_cursor_label_with_global": "γραμμή: {line} → στήλη: {column} → θέση: {position}",
    "statusbar_cursor_label_selected_with_global":
        "γραμμή: {line} → στήλη: {column} → θέση: {position} → επιλ: {selected}",
}



# Greek lexemes toolbar.py
lexemes = {
    "actions_label_new_document": "Νέο έγγραφο",
    "actions_accessible_name_new_document": "Νέο έγγραφο",
    "actions_label_open": "Άνοιγμα",
    "actions_accessible_name_open": "Άνοιγμα εγγράφου",
    "actions_label_save": "Αποθήκευση",
    "actions_accessible_name_save": "Αποθήκευση εγγράφου",
    "actions_label_save_as": "Αποθήκευση ως",
    "actions_accessible_name_save_as": "Αποθήκευση ως αντίγραφο εγγράφου",
    "actions_label_sync": "Συγχρονισμός",
    "actions_accessible_name_sync": "Συγχρονισμός εγγράφου με εξωτερική αποθήκευση",
    "actions_label_edit": "Επεξεργασία",
    "actions_accessible_name_edit": "Επεξεργασία αρχείου",
    "actions_label_source": "Πηγή",
    "actions_accessible_name_source": "Κώδικας πηγής",
    "actions_label_encrypt": "Κρυπτογράφηση",
    "actions_accessible_name_encrypt": "Κρυπτογράφηση αρχείου",
    "actions_label_decrypt": "Αποκρυπτογράφηση",
    "actions_accessible_name_decrypt": "Αποκρυπτογράφηση αρχείου",
    "actions_label_bold": "Έντονα",
    "actions_accessible_name_bold": "Μορφή έντονου κειμένου",
    "actions_label_italic": "Πλάγια",
    "actions_accessible_name_italic": "Μορφή πλάγιου κειμένου",
    "actions_label_underline": "Υπογράμμιση",
    "actions_accessible_name_underline": "Μορφή υπογεγραμμένου κειμένου",
    "actions_label_strikethrough": "Διαγράμμιση",
    "actions_accessible_name_strikethrough": "Μορφή διαγραμμισμένου κειμένου",
    "actions_label_blockquote": "Παράθεση",
    "actions_accessible_name_blockquote": "Μορφή κειμένου παράθεσης",
    "actions_label_color_picker": "Επιλογέας χρώματος",
    "actions_accessible_name_color_picker": "Επιλογή χρώματος κειμένου",
    "actions_label_ai_assistant": "Βοηθός AI",
    "actions_accessible_name_ai_assistant": "Εκκίνηση Βοηθού AI",
    "actions_label_about": "Πληροφορίες",
    "actions_accessible_name_about": "Πληροφορίες για την εφαρμογή",
    "actions_label_exit": "Έξοδος",
    "actions_accessible_name_exit": "Έξοδος από την εφαρμογή",
    "actions_label_settings": "Ρυθμίσεις",
    "actions_accessible_name_settings": "Ρυθμίσεις εφαρμογής",

    "search_input_label": "Αναζήτηση",
    "search_input_placeholder_text": "Αναζήτηση κειμένου στο έγγραφο",
    "search_input_accessible_description": "Πεδίο εισόδου αναζήτησης",
    "search_buttons_label_clear": "Καθαρισμός αναζήτησης",
    "search_buttons_accessible_name_clear": "Καθαρισμός πεδίου εισόδου αναζήτησης",
    "search_buttons_label_prev": "Προηγούμενη αναζήτηση",
    "search_buttons_accessible_name_prev": "Προηγούμενο αποτέλεσμα αναζήτησης",
    "search_buttons_label_next": "Επόμενη αναζήτηση",
    "search_buttons_accessible_name_next": "Επόμενο αποτέλεσμα αναζήτησης",
    "search_case_sensitive_accessible_name": "Διάκριση πεζών-κεφαλαίων",
    "search_case_sensitive_tooltip": "Αναζήτηση με διάκριση πεζών-κεφαλαίων",
}



# English lexemes common.py
lexemes = {
    # Common
    "module_ondevice_llm_name": "On Device LLM",

    "module_ondevice_llm_model_exception": "Model not found error: {error_msg}",
    "module_ondevice_llm_task_exception": "Cannot init model generator: {error_msg}",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Contains unit and integration tests for the related functionality.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import Qt, QObject
from PySide6.QtWidgets import QWidget, QTabWidget, QLabel, QCheckBox, QLineEdit, QSlider
from PySide6.QtWidgets import QScrollArea, QVBoxLayout, QSizePolicy, QPlainTextEdit, QComboBox, QSpinBox, QDoubleSpinBox
from PySide6.QtTest import QTest

from notolog.ui.settings_dialog import SettingsDialog
from notolog.ui.enum_combo_box import EnumComboBox
from notolog.ui.label_with_hint import LabelWithHint
from notolog.ui.dir_path_line_edit import DirPathLineEdit
from notolog.ui.file_path_line_edit import FilePathLineEdit
from notolog.ui.horizontal_line_spacer import HorizontalLineSpacer
from notolog.notolog_editor import NotologEditor
from notolog.settings import Settings
from notolog.enums.languages import Languages
from notolog.modules.modules import Modules

from ..test_enums.something_enum import SomethingEnum

from . import test_app  # noqa: F401

import pytest

from unittest.mock import MagicMock


class TestSettingsDialog:

    @pytest.fixture(scope="class", autouse=True)
    def settings_obj(self):
        """
        Use 'autouse=True' to enable automatic setup, or pass 'settings_obj' directly to main_window()
        """
        # Fixture to create and return settings instance
        settings = Settings()
        # Clear settings to be sure start over without side effects
        settings.clear()
        # Reset singleton (qa functionality)
        Settings.reload()

        yield settings

    @pytest.fixture(autouse=True)
    def modules_obj(self):
        # Fixture to create and return modules instance
        modules = Modules()
        modules.modules = []
        yield modules

    @pytest.fixture
    def main_window(self, mocker, test_app):  # noqa: F811 redefinition of unused 'test_app'
        # Force to override system language as a default
        mocker.patch.object(Languages, 'default', return_value='la')

        # Do not show actual window; return object instance only
        mocker.patch.object(NotologEditor, 'show', return_value=None)
        # Prevent resource processing, including 'process_document_images'
        mocker.patch.object(NotologEditor, 'load_content_html', return_value=None)

        # Fixture to create and return main window instance
        yield NotologEditor(screen=test_app.screens()[0])

    @pytest.fixture(autouse=True)
    def ui_obj(self, mocker, main_window):
        # No modules
        mocker.patch.object(Modules, 'get_by_extension', return_value=[])
        # Fixture to initialize object.
        ui_obj = SettingsDialog(parent=main_window)
        yield ui_obj

    @pytest.fixture(scope="function")
    def test_params_fixture(self, request):
        # Retrieve parameter values from the test request.
        param_values = request.param

        yield param_values

    @pytest.fixture(scope="function")
    def test_exp_params_fixture(self, request):
        # Retrieve parameter values from the test request.
        param_values = request.param

        yield param_values

    def test_ui_object_state(self, mocker, ui_obj: SettingsDialog, settings_obj):
        # Check app language set correctly
        assert settings_obj.app_language == 'la'

        # Check default window title
        assert ui_obj.windowTitle() == "Configurationes"

        # Retrieve callback methods for verification.
        mock_init_ui = mocker.patch.object(ui_obj, 'init_ui', wraps=ui_obj.init_ui)

        # Check the method wasn't called as no ui_obj.exec() executed
        mock_init_ui.assert_not_called()

        # Mind the Latin translation
        assert ui_obj.tab_widget.tabText(0) == 'Generale'
        assert ui_obj.tab_widget.tabText(1) == 'Editor'
        assert ui_obj.tab_widget.tabText(2) == 'Visor'
        assert ui_obj.tab_widget.tabText(3) == 'Configuratio AI'
        assert ui_obj.tab_widget.tabText(4) == ''

        # Ensure the initial tab is the first tab
        assert ui_obj.tab_widget.currentIndex() == 0
        assert isinstance(ui_obj.tab_widget.currentWidget(), QScrollArea)
        assert ui_obj.tab_widget.currentWidget().objectName() == 'settings_dialog_tab_general'
        # Test that clicking the edit button updates the editor state
        QTest.mouseClick(
            # ui_obj.tab_widget.tabBar().tabButton(1, QTabBar.ButtonPosition.LeftSide), Qt.MouseButton.LeftButton)
            # This simply targets the middle of the tab, ensuring the interaction is as expected.
            ui_obj.tab_widget.tabBar(), Qt.MouseButton.LeftButton, pos=ui_obj.tab_widget.tabBar().tabRect(1).center())
        # Check the result
        assert ui_obj.tab_widget.currentIndex() == 1
        assert isinstance(ui_obj.tab_widget.currentWidget(), QScrollArea)
        assert ui_obj.tab_widget.currentWidget().objectName() == 'settings_dialog_tab_editor_config'

        checkboxes = ui_obj.findChildren(QCheckBox)
        for checkbox in checkboxes:
            if isinstance(checkbox, QCheckBox):
                # Parse the object name in case it contains a combination of lexeme and setting keys
                _lexeme_key, setting_name = ui_obj.parse_object_name(checkbox.objectName())
                if setting_name == 'show_main_menu':
                    # Either Qt.CheckState.Checked or Qt.CheckState.Unchecked
                    prev_state = checkbox.checkState()
                    QTest.mouseClick(checkbox, Qt.MouseButton.LeftButton)
                    assert checkbox.checkState() != prev_state

    @pytest.mark.parametrize(
        "test_params_fixture, test_exp_params_fixture",
        [
            ('', ['', '']),
            (':', ['', '']),
            ('obj1_name', ['obj1_name', 'obj1_name']),
            ('obj1_name:setting1_name', ['obj1_name', 'setting1_name']),
            (':setting2_name', ['', 'setting2_name']),
        ],
        indirect=True
    )
    def test_parse_object_name(self, ui_obj: SettingsDialog, test_params_fixture, test_exp_params_fixture):
        """
        Covers:
        - SettingsDialog.parse_object_name
        """

        # Extract the test method name and expected parameters
        test_object_name = test_params_fixture
        exp_result = test_exp_params_fixture

        # Run the method under test
        result = ui_obj.parse_object_name(str(test_object_name))

        assert result == exp_result

    @pytest.mark.parametrize(
        "test_params_fixture, test_exp_params_fixture",
        [
            ('get_general_fields', 'settings_dialog_tab_general'),
            ('get_editor_config_fields', 'settings_dialog_tab_editor_config'),
            ('get_viewer_config_fields', 'settings_dialog_tab_viewer_config'),
            ('get_ai_config_fields', 'settings_dialog_tab_ai_config'),
        ],
        indirect=True
    )
    def test_get_tab_fields(self, mocker, ui_obj: SettingsDialog, test_params_fixture, test_exp_params_fixture):
        """
        Covers:
        - SettingsDialog.get_general_fields
        - SettingsDialog.get_editor_config_fields
        - SettingsDialog.get_viewer_config_fields
        - SettingsDialog.get_ai_config_fields
        """

        # Extract the test method name and expected parameters
        test_method_name = test_params_fixture
        exp_tab_name = test_exp_params_fixture

        # Mock the tab widget (QTabWidget instance)
        mock_tab_widget = MagicMock(spec=QTabWidget)
        mock_mock_tab_widget_add_tab = mocker.patch.object(mock_tab_widget, 'addTab')

        # Assign the mocked tab widget to the testing object
        ui_obj.tab_widget = mock_tab_widget

        # Mock QVBoxLayout's addWidget method
        mock_vbox_layout_add_widget = mocker.patch.object(QVBoxLayout, 'addWidget', return_value=None)

        # Mock the tab's main widget's setObjectName method
        mock_tab_widget_set_object_name = mocker.patch.object(QWidget, 'setObjectName')

        # Mock the scroll area containing the main tab widget
        mock_scroll_area_set_widget = mocker.patch.object(QScrollArea, 'setWidget')

        # Run the method under test
        test_method = getattr(ui_obj, str(test_method_name))
        assert callable(test_method), f"Method {test_method_name} is not callable."
        result = test_method()

        # Assert the result is of the expected type
        assert isinstance(result, list), "Expected the result to be a list."

        # Check that setObjectName methods were called with the expected parameters
        mock_tab_widget_set_object_name.assert_called_once_with(exp_tab_name)

        # Assert that the tab's main widget was added to the scroll area
        mock_scroll_area_set_widget.assert_called_once()
        assert 'QWidget' in str(mock_scroll_area_set_widget.call_args), "Expected a QWidget in the call arguments."

        # Verify that the tab was added to the tab widget
        mock_mock_tab_widget_add_tab.assert_called_once()

        # Validate the result by examining one of its dictionary values
        config_item = result.pop(0)
        assert 'callback' in config_item, "Expected 'callback' in the configuration item."
        assert callable(config_item['callback']), "Expected 'callback' to be callable."

        # Test the callback in the configuration
        mock_obj = MagicMock(spec=QObject)
        config_item['callback'](mock_obj)

        # Verify that the layout processed the callback and added the config's value to it
        mock_vbox_layout_add_widget.assert_called_once_with(mock_obj, alignment=Qt.AlignmentFlag.AlignTop)

    @pytest.mark.parametrize(
        "test_params_fixture, test_exp_params_fixture",
        [
            ({"type": HorizontalLineSpacer},
             (HorizontalLineSpacer, lambda v: True)),
            ({"type": QWidget, "size_policy": (QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)},
             (QWidget, lambda v: True)),
            ({"type": QCheckBox, "name": "obj1"},
             (QCheckBox, lambda v: v.objectName() == "obj1")),
            ({"type": QLineEdit, "placeholder_text": "text1"},
             (QLineEdit, lambda v: v.placeholderText() == "text1")),
            ({"type": QLabel, "props": {"setProperty": ("class", "class1")}},
             (QLabel, lambda v: v.property("class") == "class1")),
            ({"type": QSlider, "props": {"setTickInterval": 123}},
             (QSlider, lambda v: v.property("tickInterval") == 123)),
            ({"type": QPlainTextEdit, "text_lines": 3},
             (QPlainTextEdit, lambda v: v.height() > 1)),
            ({"type": QSpinBox, "props": {"setMinimum": 1, "setMaximum": 65536}},
             (QSpinBox, lambda v: (v.property("minimum") == 1, v.property("maximum") == 65536))),
            ({"type": QDoubleSpinBox, "props": {"setMinimum": 0.0, "setMaximum": 9.9}},
             (QDoubleSpinBox, lambda v: (v.property("minimum") == 0.0, v.property("maximum") == 9.9))),
            ({"type": QComboBox, "callback": lambda obj: obj.setProperty('param1', 'value1')},
             (QComboBox, lambda v: v.property('param1') == 'value1')),
            ({"type": EnumComboBox, "args": [
                sorted(SomethingEnum, key=lambda member: (not member.is_default, str(member.value)))]},
             (EnumComboBox, lambda v: (v.count() == 3, isinstance(v.currentIndex(), int)))),
            ({"type": LabelWithHint, "kwargs": {"tooltip": ("lexeme1_key", "lexeme1_text")}, "text": "text1"},
             (LabelWithHint, lambda v: (v.property("tooltip_lexeme") == "lexeme1_key", v.text == "text1"))),
            ({"type": DirPathLineEdit, "kwargs": {"default_directory": "dir1"}},
             (DirPathLineEdit, lambda v: (v.default_directory == "dir1"))),
            ({"type": FilePathLineEdit, "kwargs": {"ext_filter": "ext1"}},
             (FilePathLineEdit, lambda v: (v.ext_filter == "ext1"))),
        ],
        indirect=True
    )
    def test_create_setting_field(self, mocker, ui_obj: SettingsDialog, test_params_fixture, test_exp_params_fixture):
        # Extract test and expected params
        test_config = test_params_fixture
        exp_result_type, exp_check = test_exp_params_fixture

        # Run the method under test
        result_obj = ui_obj.create_setting_field(dict(test_config))

        # Assert the result is of the expected type
        assert isinstance(result_obj, exp_result_type)  # noqa

        # Test the callable check method
        assert callable(exp_check)
        assert exp_check(result_obj)



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Encryption helper class.
- Functionality: Encrypts/decrypts file data with a symmetric encryption algorithm.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
from cryptography.fernet import Fernet

from .enc_password import EncPassword

from . import Settings

import logging
import secrets
import base64

from typing import Union


class EncHelper:
    """
    Adjust the number of iterations for more security.
    Remember to re-encrypt the source.
    """
    DEFAULT_ITERATIONS = 768000

    def __init__(self, enc_password: EncPassword = None, salt: str = None, iterations: int = None):
        super().__init__()

        self.logger = logging.getLogger('enc_helper')

        self.enc_password = enc_password
        self.salt = salt
        self.iterations = iterations

        if self.enc_password is not None:
            self.password = self.enc_password.password if self.enc_password.password is not None else ''
        else:
            self.logger.warning('No password provided! Unsecure result')
            self.enc_password = EncPassword()
            self.enc_password.password = self.password = ''

        if type(self.password) is not bytes:
            self.password = self.password.encode()

        if self.salt is None:
            self.logger.warning('No salt provided! Random value generated')
            self.salt = self.generate_salt()

        if type(self.salt) is not bytes:
            self.salt = self.salt.encode()

        if self.iterations is None:
            self.logger.warning('No iterations provided! Default value fallback')
            self.iterations = self.__class__.get_default_iterations()

        # Derive key from password
        key = self.generate_key_from_password()
        encoded_key = base64.urlsafe_b64encode(key)
        # Fernet uses only the first 128 bits (16 bytes) of the decoded key for AES encryption.
        self.cipher_suite = Fernet(encoded_key)

        self.key = None

    def generate_key_from_password(self) -> bytes:
        """
        Generate a key from the password

        Password-Based Key Derivation Function 2 (PBKDF2) implementation.
        The param length=32 is the length of the derived key.
        """
        kdf = PBKDF2HMAC(
            algorithm=hashes.SHA256(),
            iterations=self.iterations,
            salt=self.salt,
            length=32  # 32 bytes = 256 bits
        )

        self.key = kdf.derive(self.password)

        return self.key

    @staticmethod
    def generate_key() -> bytes:
        """
        Generate a key without password, just in case
        """
        return Fernet.generate_key()

    def encrypt_data(self, source_data: bytes) -> Union[bytes, None]:
        """
        Encrypt the data.
        This method can be called when no real password was set due cancellation.
        """
        if not self.is_password_valid():
            return None
        return self.cipher_suite.encrypt(source_data)

    def decrypt_data(self, encrypted_data: bytes) -> Union[bytes, None]:
        """
        Decrypt the data.
        This method can be called when no real password was set due to cancellation.
        """
        if not self.is_password_valid():
            return None
        return self.cipher_suite.decrypt(encrypted_data)

    def is_password_valid(self) -> bool:
        """
        Check either the password is valid or not
        """
        return self.password is not None and len(self.password) > 0

    @staticmethod
    def generate_salt(length: int = 32) -> str:
        """
        Generate new salt static method
        """
        return secrets.token_urlsafe(length)

    @staticmethod
    def get_default_iterations() -> int:
        # Allow update default value and store it in settings
        settings = Settings()
        # Return number of iterations from settings, or use the default fallback to update it
        if not settings.enc_iterations:
            # If settings value wasn't initiated
            settings.enc_iterations = EncHelper.DEFAULT_ITERATIONS
        return int(settings.enc_iterations)



# Korean lexemes settings_dialog.py
lexemes = {
    "tab_module_llama_cpp_config": "모듈 llama.cpp",

    "module_llama_cpp_config_label": "모듈 llama.cpp",
    "module_llama_cpp_config_path_label": "모델 위치",
    "module_llama_cpp_config_path_input_placeholder_text": "모델 경로 선택 또는 입력",
    "module_llama_cpp_config_path_input_accessible_description":
        "로컬 모델의 경로를 지정하는 선택기가 있는 입력 필드입니다. GGUF 형식의 모델을 지원하며,\n"
        "GGML 및 GGML 기반 실행기와 함께 사용되는 모델을 저장하기 위해 최적화된 이진 파일 형식입니다.",
    "module_llama_cpp_config_path_input_filter_text": "GGUF 파일",

    "module_llama_cpp_config_context_window_label": "콘텍스트 윈도우 크기",
    "module_llama_cpp_config_context_window_input_accessible_description":
        "모델이 응답을 생성하는 데 고려하는 토큰 수를 설정합니다. 얼마나 많은 이전 콘텍스트가 사용되는지를 제어합니다.",

    "module_llama_cpp_chat_formats_label": "채팅 형식",
    "module_llama_cpp_chat_formats_combo_placeholder_text": "채팅 형식 선택",
    "module_llama_cpp_chat_formats_combo_accessible_description":
        "모델 대화에 사용되는 형식을 선택하기 위한 드롭다운 메뉴입니다.",

    "module_llama_cpp_config_system_prompt_label": "시스템 프롬프트",
    "module_llama_cpp_config_system_prompt_edit_placeholder_text": "시스템 프롬프트 텍스트를 입력하세요",
    "module_llama_cpp_config_system_prompt_edit_accessible_description":
        "모델 응답을 안내하는 시스템 프롬프트를 입력하기 위한 텍스트 필드입니다.",

    "module_llama_cpp_config_response_temperature_label": "응답 온도: {temperature}",
    "module_llama_cpp_config_response_temperature_input_accessible_description":
        "모델 응답의 무작위성을 조절합니다. 높은 값은 더 다양한 출력을 생성하며,\n"
        "낮은 값은 더 예측 가능한 응답을 초래합니다.",

    "module_llama_cpp_config_response_max_tokens_label": "응답당 최대 토큰 수",
    "module_llama_cpp_config_response_max_tokens_input_accessible_description":
        "실제 콘텍스트 윈도우 한계까지 모델 응답에서 토큰 수를 제한합니다.\n"
        "제로 값은 콘텍스트 윈도우의 용량을 가정합니다.",

    "module_llama_cpp_config_prompt_history_size_label": "프롬프트 기록 크기",
    "module_llama_cpp_config_prompt_history_size_input_accessible_description":
        "시스템에 의해 참조용으로 유지되는 프롬프트 기록의 항목 수를 제어합니다.\n"
        "제로 값은 무제한 항목을 허용합니다."
}



# Georgian lexemes common.py
lexemes = {
    "module_llama_cpp_name": "მოდული llama.cpp",

    "module_llama_cpp_model_exception": "მოდელი ვერ მოიძებნა შეცდომა: {error_msg}",
    "module_llama_cpp_task_exception": "მოდელის გენერატორის ინიციალიზაცია ვერ ხდება: {error_msg}",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Extends QWidget functionality to provide vertical UI spacer.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtWidgets import QFrame


class VerticalLineSpacer(QFrame):
    """
    Vertical line separator for the status bar.
    """

    def __init__(self):
        super(VerticalLineSpacer, self).__init__()

        # https://doc.qt.io/qt-6/qframe.html#Shape-enum
        self.setFrameShape(QFrame.Shape.VLine)

        # https://doc.qt.io/qt-6/qframe.html#Shadow-enum
        self.setFrameShadow(QFrame.Shadow.Raised)

        self.setLineWidth(1)



# Italian lexemes main_menu.py
lexemes = {
    "group_file_label": "File",
    "group_edit_label": "Modifica",
    "group_tools_label": "Strumenti",
    "group_help_label": "Aiuto",

    "actions_file_label_new_document": "Nuovo documento",
    "actions_file_accessible_name_new_document": "Nuovo documento",
    "actions_file_label_open": "Apri",
    "actions_file_accessible_name_open": "Apri documento",
    "actions_file_label_save": "Salva",
    "actions_file_accessible_name_save": "Salva documento",
    "actions_file_label_save_as": "Salva come",
    "actions_file_accessible_name_save_as": "Salva come copia del documento",
    "actions_file_label_settings": "Impostazioni",
    "actions_file_accessible_name_settings": "Impostazioni dell'app",
    "actions_file_label_reset_settings": "Reimposta impostazioni",
    "actions_file_accessible_name_reset_settings": "Reimposta impostazioni",
    "actions_file_label_exit": "Esci",
    "actions_file_accessible_name_exit": "Esci dall'applicazione",

    "actions_edit_edit_mode": "Modifica",
    "actions_edit_accessible_name_edit_mode": "Modifica file",
    "actions_edit_source_mode": "Sorgente",
    "actions_edit_accessible_name_source_mode": "Codice sorgente",
    "actions_edit_bold": "Grassetto",
    "actions_edit_accessible_name_bold": "Formato testo grassetto",
    "actions_edit_italic": "Corsivo",
    "actions_edit_accessible_name_italic": "Formato testo corsivo",
    "actions_edit_underline": "Sottolineato",
    "actions_edit_accessible_name_underline": "Formato testo sottolineato",
    "actions_edit_strikethrough": "Barrato",
    "actions_edit_accessible_name_strikethrough": "Formato testo barrato",
    "actions_edit_blockquote": "Citazione",
    "actions_edit_accessible_name_blockquote": "Formato testo citazione",

    "actions_tools_ai_assistant": "Assistente IA",
    "actions_tools_accessible_name_ai_assistant": "Avvia Assistente IA",
    "actions_tools_color_picker": "Selettore Colore",
    "actions_tools_accessible_name_color_picker": "Scegli un colore per il testo",

    "actions_help_label_md_syntax": "Sintassi Markdown",
    "actions_help_accessible_name_md_syntax": "Sintassi Markdown",
    "actions_help_label_check_for_updates": "Controlla aggiornamenti",
    "actions_help_accessible_name_check_for_updates": "Controlla aggiornamenti",
    "actions_help_label_bug_report": "Segnala bug",
    "actions_help_accessible_name_bug_report": "Invia segnalazione di bug",
    "actions_help_label_about": "Informazioni",
    "actions_help_accessible_name_about": "Informazioni",
}



# English lexemes toolbar.py
lexemes = {
    "actions_label_new_document": "New document",
    "actions_accessible_name_new_document": "New document",
    "actions_label_open": "Open",
    "actions_accessible_name_open": "Open document",
    "actions_label_save": "Save",
    "actions_accessible_name_save": "Save document",
    "actions_label_save_as": "Save as",
    "actions_accessible_name_save_as": "Save as document copy",
    "actions_label_sync": "Sync",
    "actions_accessible_name_sync": "Sync document with an external storage",
    "actions_label_edit": "Edit",
    "actions_accessible_name_edit": "Edit file",
    "actions_label_source": "Source",
    "actions_accessible_name_source": "Source code",
    "actions_label_encrypt": "Encrypt",
    "actions_accessible_name_encrypt": "Encrypt file",
    "actions_label_decrypt": "Decrypt",
    "actions_accessible_name_decrypt": "Decrypt file",
    "actions_label_bold": "Bold",
    "actions_accessible_name_bold": "Bold text format",
    "actions_label_italic": "Italic",
    "actions_accessible_name_italic": "Italic text format",
    "actions_label_underline": "Underline",
    "actions_accessible_name_underline": "Underline text format",
    "actions_label_strikethrough": "Strikethrough",
    "actions_accessible_name_strikethrough": "Strikethrough text format",
    "actions_label_blockquote": "Blockquote",
    "actions_accessible_name_blockquote": "Blockquote text format",
    "actions_label_color_picker": "Color Picker",
    "actions_accessible_name_color_picker": "Choose a text color",
    "actions_label_ai_assistant": "AI Assistant",
    "actions_accessible_name_ai_assistant": "Launch AI Assistant",
    "actions_label_about": "About",
    "actions_accessible_name_about": "About the App",
    "actions_label_exit": "Exit",
    "actions_accessible_name_exit": "Exit the App",
    "actions_label_settings": "Settings",
    "actions_accessible_name_settings": "App settings",

    "search_input_label": "Search",
    "search_input_placeholder_text": "Search text in the document",
    "search_input_accessible_description": "Search input field",
    "search_buttons_label_clear": "Clear search",
    "search_buttons_accessible_name_clear": "Clear search input field",
    "search_buttons_label_prev": "Previous search",
    "search_buttons_accessible_name_prev": "Previous search result",
    "search_buttons_label_next": "Next search",
    "search_buttons_accessible_name_next": "Next search result",
    "search_case_sensitive_accessible_name": "Case sensitive",
    "search_case_sensitive_tooltip": "Case sensitive search",
}



# Japanese lexemes settings_dialog.py
lexemes = {
    "tab_openai_api_config": "OpenAI API",

    "module_openai_api_label": "OpenAI API",
    "module_openai_api_url_label": "APIのURL",
    "module_openai_api_url_input_placeholder_text": "APIのURL",
    "module_openai_api_url_input_accessible_description":
        "OpenAI APIのURLはAPIエンドポイントのアドレスであり、サービスやバージョンによって異なる場合があります。\n"
        "AIアシスタントは、会話型チャット機能またはテキスト補完用のエンドポイントを使用します。\n"
        "最新のURLを取得するには、OpenAI APIの公式ドキュメントを参照してください。",
    "module_openai_api_key_label": "APIキー",
    "module_openai_api_key_input_placeholder_text": "APIキー",
    "module_openai_api_key_input_accessible_description":
        "OpenAI APIキーは、APIエンドポイントへのリクエストを認証するために使用される秘密のトークンです。",
    "module_openai_api_supported_models_label": "対応モデル",
    "module_openai_api_model_names_combo_placeholder_text": "モデルを選択",
    "module_openai_api_model_names_combo_accessible_description":
        "チャット会話用に対応するモデルから選択します。",

    "module_openai_api_base_system_prompt_label": "システムプロンプト",
    "module_openai_api_base_system_prompt_edit_placeholder_text": "各リクエストに先立つ基本システムプロンプト",
    "module_openai_api_base_system_prompt_edit_accessible_description":
        "各リクエストに先立つ基本システムプロンプト。\n"
        "通常、指示や役割の特性を含むプレーンテキストです。",

    "module_openai_api_base_response_temperature_label": "温度: {temperature}",
    "module_openai_api_base_response_temperature_input_accessible_description":
        "モデルの応答のランダム性を調整します。高い値はより多様な出力を生成し、\n"
        "低い値は応答をより予測可能にします。",

    "module_openai_api_base_response_max_tokens_label": "最大応答トークン数",
    "module_openai_api_base_response_max_tokens_input_accessible_description":
        "応答で受け取るトークンの最大数を設定します。これには単語や句読点が含まれ、\n"
        "出力の長さを制御します。",

    "module_openai_api_config_prompt_history_size_label": "プロンプト履歴のサイズ",
    "module_openai_api_config_prompt_history_size_input_accessible_description":
        "システムが参照用に保持するプロンプト履歴のエントリ数を制御します。\n"
        "0の値は無制限のエントリを許可します。"
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Document view processor to support additional modifications in the resulting text.
- Functionality: Supports expandable markdown blocks, such as <details> and <summary>.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import Qt, QUrl
from PySide6.QtWidgets import QTextBrowser
from PySide6.QtGui import QTextCursor, QSyntaxHighlighter, QTextCharFormat

import logging
import re
import base64

from typing import TYPE_CHECKING, Union

from .settings import Settings
from .highlight.view_highlighter import ViewHighlighter
from .lexemes.lexemes import Lexemes

if TYPE_CHECKING:
    from PySide6.QtCore import QObject  # noqa: F401


class ViewProcessor:
    """
    View mode pre-result procession by modifying loaded QTextDocument
    """

    zero_width_space = '\u200B'  # '&#8203;' or '​'

    def __init__(self, highlighter: Union[QSyntaxHighlighter, ViewHighlighter]):
        """
        Args:
            highlighter (Union[QSyntaxHighlighter, ViewHighlighter]):
            Highlighter that holds document to apply any modifications.
        """
        self.highlighter = highlighter
        self.doc = self.highlighter.document()

        self.settings = Settings()

        self.logger = logging.getLogger('view_processor')

        # Load lexemes for selected language and scope
        self.lexemes = Lexemes(self.settings.app_language, default_scope='common')

        self.logger.debug('Characters count %d' % self.doc.characterCount())

        self.blocks = []
        self.blocks_start = []
        self.blocks_end = []

        # self.doc.contentsChanged.connect(self.process)

        cursor = QTextCursor(self.doc)
        self.cursor_pos_orig = cursor.position()

    def restore_cursor_pos(self):
        cursor = QTextCursor(self.doc)
        # Restore original cursor position
        cursor.setPosition(self.cursor_pos_orig, QTextCursor.MoveMode.MoveAnchor)
        parent = self.doc.parent()  # type: QObject
        parent_widget = parent.get_view_widget()  # type: QTextBrowser
        parent_widget.setTextCursor(cursor)

    @staticmethod
    def replace_tags(content, replacements) -> str:
        """
        Function to perform replacements.

        Args:
            content (str): Source content.
            replacements (dict): Replacements mapping.
        """
        for pattern, replacement in replacements.items():
            content = re.sub(pattern, replacement, content)
        return content

    def pre_md_process(self, content: str):
        """
        Replaces expandable tokens from html-like to the meta-like,
        to avoid ignoring them upon markdown conversion.

        Args:
            content (str): Source content.

        Returns:
            str: Pre-processed content.
        """

        # Define the replacements in a dictionary with regex patterns
        forward_replacements = {
            r'<details([^>]*?)>': r'[details\1]',
            r'</details>': '[/details]',
            r'<summary([^>]*?)>': r'[summary\1]',
            r'</summary>': '[/summary]',
        }

        return self.replace_tags(content, forward_replacements)

    def post_md_process(self, content: str = None):
        """
        Revert the replacements of the expandable tokens from meta-like back to the html-like,
        to allow to recognize them upon html-rendering.

        Args:
            content (str, optional): Source content.

        Returns:
            str: Post-processed content.
        """

        # Whither to get the content from the doc or not
        doc_processing = True if content is None else False
        if doc_processing:
            content = self.doc.toPlainText()

        # Define the backward replacements in a dictionary
        backward_replacements = {
            r'\[details([^>]*?)\]': r'<details\1>',
            r'\[/details\]': '</details>',
            r'\[summary([^>]*?)\]': r'<summary\1>',
            r'\[/summary\]': '</summary>',
        }

        # Perform the replacements
        post_processed_text = self.replace_tags(content, backward_replacements)

        if doc_processing:
            self.doc.setPlainText(post_processed_text)

        return post_processed_text

    def process(self):
        self.blocks.clear()
        cursor = QTextCursor(self.doc)
        cursor.setPosition(0)

        def process_block(cursor_internal: QTextCursor) -> None:
            """
            Find open/close tokens in block.
            @param cursor_internal: active cursor
            @return: None
            """
            block = cursor_internal.block()
            pattern = r"<details.*?>"
            # Find all matches of the pattern in provided string
            matches = re.finditer(pattern, block.text())
            for match in matches:
                _open_start = match.start()
                _open_end = match.end()
                _open_length = _open_end - _open_start
                self.blocks_start.append((block.position() + _open_start, _open_length))
            pattern = r"</details>"
            # Find all matches of the pattern in provided string
            matches = re.finditer(pattern, block.text())
            for match in matches:
                _close_start = match.start()
                _close_end = match.end()
                _close_length = _close_end - _close_start
                self.blocks_end.append((block.position() + _close_start, _close_length))

        cursor.movePosition(QTextCursor.MoveOperation.Start)
        # Iterate through each block
        while not (cursor.atEnd()
                   # cursor.atEnd() not enough as the cursor unable to move next block at the very end
                   or (cursor.block().blockNumber() > 0
                       and cursor.block().blockNumber() == self.doc.blockCount() - 1)):
            # Process block
            process_block(cursor)
            # Move to the next block
            cursor.movePosition(QTextCursor.MoveOperation.NextBlock)

            # To avoid never ending cycle when only one block is available
            if self.doc.blockCount() == 1:
                break

        # Process block at the very end
        process_block(cursor)

        # Reset warning if exists
        self.doc.parent().statusbar.show_warning(visible=False)
        # Check open/close tokens match
        if len(self.blocks_start) != len(self.blocks_end):
            # Show error sign at status bar
            self.doc.parent().statusbar.show_warning(
                visible=True,
                tooltip=self.lexemes.get('expandable_block_open_close_tags_mismatch_warning'))

        res = []
        self.blocks_start.reverse()
        # Enumerate by end token to find the closest start token
        for index, value in enumerate(self.blocks_end):
            close_start, close_length = value
            if index not in res:
                """
                Extra data allows independent processing of tokens and their content:
                o - open, start position
                ol - open length, start token length, say <details> length is 9
                c - close, close position
                cl - close length, close token length (the same approach as for open length)
                """
                res.insert(index, {'o': None, 'ol': None, 'c': close_start, 'cl': close_length, 'g': None, 'l': 0})
            # Enumerate start token list backward unsetting matched element
            for _index, _value in enumerate(self.blocks_start):
                open_start, open_length = _value
                if _value < value:
                    res[index].update({'o': open_start, 'ol': open_length})
                    del self.blocks_start[_index]
                    break

        self.logger.debug('Unsorted open-close mapping %s' % res)

        self.blocks = res

        if self.blocks:
            self.sort_blocks()
            self.collapse_blocks()

        # Restore original cursor position
        self.restore_cursor_pos()

    def sort_blocks(self):
        self.logger.debug('Sorting elements by group and nesting level...')

        self.blocks.sort(key=lambda x: x['o'] if 'o' in x and x['o'] is not None else 0)

        self.logger.debug('Sorted result of the open-close mapping %s' % self.blocks)

        group = 0
        for i, _data in enumerate(self.blocks):
            if not _data['l']:
                # Update group only for each root element
                group = i
                self.blocks[i].update({'g': group})
            # Iterate through remaining elements to find possible nesting
            for j, _other_data in enumerate(self.blocks[i + 1:], start=i + 1):
                if _data['o'] is None or _other_data['o'] is None or _data['c'] is None or _other_data['c'] is None:
                    # Show error sign at status bar
                    self.doc.parent().statusbar.show_warning(
                        visible=True,
                        tooltip=self.lexemes.get('expandable_block_open_close_tags_mismatch_warning'))
                    return
                # Check if other element is nested within the current element
                if _data['o'] < _other_data['o'] and _data['c'] > _other_data['c']:
                    self.blocks[j].update({'l': (_data['l'] + 1 if 'l' in _data else 1), 'g': group})

        """
        Sort elements by:
        1. By group first (nested elements or singles)
        2. Nesting level (more means deeper nesting)
        3. Open token position (DESC order)
        """
        self.blocks.sort(key=lambda x: (-x['g'] if x['g'] is not None else 0,
                                        -x['l'] if x['l'] is not None else 0,
                                        -x['o'] if x['o'] is not None else 0))  # reverse=True

        self.logger.debug('Final result of the open-close mapping with nesting value %s' % self.blocks)

    def collapse_blocks(self):
        cursor = QTextCursor(self.doc)
        # Keep prev states to check within a loop
        prev_group = None
        prev_level = 0
        replacement_text_len = 0
        replacement_text_lengths = {}
        for pair in self.blocks:
            """
            Blocks being processed in a particular order.
            The sorting applied at the end of the sort_blocks() method.
            """
            pos_group = pair['g']
            pos_level = pair['l']
            # When start processing another group of nested (or single) elements
            if pos_group != prev_group:
                prev_group = pos_group
                replacement_text_len = 0
                replacement_text_lengths = {}
            # Previous processed pair['l'] value have to be set
            if prev_level not in replacement_text_lengths:
                replacement_text_lengths[prev_level] = 0
            """
            Text has been replaced with a new one, different length. Include it in length calculation.
            The value is from the previous iteration but can be modified at this one if the conditions match.

            If the previous nesting level is the same as a current one re-write the length correction then.
            """
            replacement_text_lengths[prev_level] = replacement_text_len

            if pair['o'] is None or pair['c'] is None:
                self.logger.debug('Notice: Incomplete open/close tag data %s' % pair)
                return

            """
            The most nested blocks being processed first, thus the close token correction,
            as the open token doesn't change its position.
            """
            pos_open = pair['o']
            pos_close = (pair['c'] + pair['cl']  # text + closing token together to get the very end of the text
                         # Combined length correction of all nested elements inside (previous level), say each ones:
                         # [0]...[1][2][/2][/1]...[1][/1]...[/0]
                         + (replacement_text_lengths[prev_level] if pos_level == 0
                            # Replacements on the same level shouldn't interfere each other
                            else (replacement_text_len if pos_level != prev_level else 0)))

            self.logger.debug('Cursor position open: %d, close: %d, level: %d ' % (pos_open, pos_close, pos_level))

            cursor.setPosition(pos_open)
            cursor.setPosition(pos_close, QTextCursor.MoveMode.KeepAnchor)
            selected_text = cursor.selectedText()
            if selected_text:
                self.logger.debug('Block text[%d][%d]: %s' % (pos_group, pos_level, selected_text))

                cursor.removeSelectedText()

                """
                Select extra space characters before and after to strip the cursor's text the way where the summary's
                content starts right after the details tag.
                """
                pattern = r"[\s]*?<summary.*?>(.*?)<\/summary>[\s]*?"
                match = re.search(pattern, selected_text, flags=re.DOTALL | re.MULTILINE | re.UNICODE)
                summary = self.lexemes.get('expandable_block_default_title')
                if match:
                    summary = f"{match.group(1).strip()}"
                    pos_open_summary = match.start()
                    pos_close_summary = match.end()
                    selected_text = selected_text[:pos_open_summary] + selected_text[pos_close_summary:]

                # There is a possible situation where the formatted html may become invalid.
                # Say, "<p>...\n<details> </p>\n\n<summary>".
                # That's because of markdown lib may process paragraph that way.
                # Double check it with a something like exp below, but keep in mind the open tag is also exists:
                # pattern = r"<details.*?>[\s]*?(</p>)"

                selected_text = selected_text.strip()
                # Post-processing converts meta-like tokens '[details]' back to their HTML-like equivalents '<details>'
                # before the actual encoding process.
                selected_text = self.post_md_process(selected_text)
                encoded_text = base64.b64encode(selected_text.encode('utf-8'))
                encoded_summary = base64.b64encode(summary.encode('utf-8'))
                """
                Insert as a TEXT here, not as an HTML or tags will be stripped after.
                Place extra spaces on each side to allow smooth anchor detection upon click.

                Note:
                    - Do not use id="..." it makes anchor detection unstable, use data attributes instead.
                    - Avoid using <div> as it produces extra <p> after conversion.
                """
                replacement_text = ('<p class="_ds_expand">{}'
                                    # <a>...</a> will be replaced with a table upon a click
                                    # zero-width space before expandable anchor to prevent never-ending anchor sequence
                                    '<a href="expandable:{}#{}" data-group="{}" data-level="{}">'
                                    '<span class="_ds_expand_pointer">▼</span>&nbsp;{}</a>'
                                    '</p>'
                                    .format(self.zero_width_space, encoded_text.decode('utf-8'),
                                            encoded_summary.decode('utf-8'), pos_group, pos_level, summary))
                replacement_text_len += len(replacement_text) - (pos_close - pos_open)
                cursor.insertText(replacement_text)
                self.logger.debug('Replacement text: %s' % replacement_text)

                # Save current group and level to check them over again within the next iteration.
                prev_group = pos_group
                prev_level = pos_level

    def mouse_click_event(self, event):
        parent = self.doc.parent()  # type: QObject
        parent_widget = parent.get_view_widget()  # type: QTextBrowser

        # Parent actions, like select text with a cursor
        QTextBrowser.mousePressEvent(parent_widget, event)

        cursor = parent_widget.cursorForPosition(event.pos())

        if event.button() == Qt.MouseButton.LeftButton:
            anchor = parent_widget.anchorAt(event.pos())
            if anchor.startswith('expandable'):
                return self.expandable_click_event(QUrl(anchor), cursor)
            elif anchor.startswith('collapsible'):
                return self.collapsible_click_event(QUrl(anchor), cursor)
            else:
                # cursor.select(QTextCursor.SelectionType.WordUnderCursor)
                # text = cursor.selectedText()
                # self.logger.debug(f'Click context: {text}')

                cursor.clearSelection()
                parent_widget.setTextCursor(cursor)

    def process_anchor_label(self, cursor):
        # Start from the clicked position
        cursor.select(QTextCursor.SelectionType.WordUnderCursor)

        self.logger.debug('Cursor data, pos: %d, anchor: %s, text: %s'
                          % (cursor.position(), cursor.anchor(), cursor.selectedText()))

        i = 0
        # Move to the anchor caption's start position
        prev_pos = cursor.position()
        while (cursor.charFormat().isAnchor()
               and (i := i + 1)
               # Ensure that zero-width spaces are not affected.
               and cursor.document().characterAt(cursor.position() - 1) != self.zero_width_space):
            cursor.movePosition(QTextCursor.MoveOperation.PreviousCharacter, QTextCursor.MoveMode.MoveAnchor)
            # Check the cursor position is changing
            if prev_pos == cursor.position():
                break
            prev_pos = cursor.position()

        # Move to the anchor caption's end position
        prev_pos = cursor.position()
        # 0 if the mouse is clicked on the very first character '​▼'
        while ((i >= 0 or cursor.charFormat().isAnchor())
               and cursor.document().characterAt(cursor.position() + 1) != self.zero_width_space):
            i -= 1
            cursor.movePosition(QTextCursor.MoveOperation.NextCharacter, QTextCursor.MoveMode.KeepAnchor)
            # Check the cursor position is changing
            if cursor.atBlockEnd() or prev_pos == cursor.position():
                break
            prev_pos = cursor.position()

        # Remove all trailing spaces and save the cursor position
        pre_cursor_char = cursor.document().characterAt(cursor.position() - 1)
        pattern = r"([\s]+)"
        match = re.search(pattern, pre_cursor_char, flags=re.DOTALL | re.UNICODE)
        if match and match.group(1):
            j = 0
            while j < len(match.group(1)):
                cursor.movePosition(QTextCursor.MoveOperation.PreviousCharacter, QTextCursor.MoveMode.KeepAnchor)
                j += 1

        self.logger.debug(f'Anchor summary text: "{cursor.selectedText()}"')

        cursor.removeSelectedText()

        return cursor

    def expandable_click_event(self, url: QUrl, cursor: QTextCursor):
        # cursor.charFormat().anchorHref()
        self.logger.debug('Expandable anchor clicked, scheme: %s, path: %s, fragment: %s'
                          % (url.scheme(), url.path(), url.fragment()))

        # This may cause left-right click issue when either the start or end position of token is clicked
        # cursor.movePosition(QTextCursor.MoveOperation.PreviousCharacter, QTextCursor.MoveMode.KeepAnchor)

        cursor = self.process_anchor_label(cursor)
        # start_pos = cursor.position()

        encoded_text = url.path()
        self.logger.debug('Text to decode: {%s}', encoded_text)

        decoded_text = (base64.b64decode(encoded_text.encode('utf-8'))
                        .decode('utf-8')
                        .strip())

        encoded_summary = url.fragment()
        self.logger.debug('Summary to decode: {%s}', encoded_summary)
        decoded_summary = (base64.b64decode(encoded_summary.encode('utf-8'))
                           .decode('utf-8')
                           .strip())

        self.logger.debug('Decoded text: {%s}' % decoded_text)

        # Set default format for inserted text
        default_format = QTextCharFormat()
        cursor.setCharFormat(default_format)
        """
        - Insert decoded text, new line either "\n" or "<br/>" depending on insertText() or insertHtml() correspondingly.
        - Allow extra space as space symbols could be stripped during conversion.
        - Avoiding invisible separator allows the whole block to be an anchor and being replaced at once.
        """
        collapsible_html = ('<table class="_n_details _ds_collapse">'
                            '<tr><th class="_n_details_summary">'
                            '<a href="collapsible:{}#{}" class="_ds_collapse_summary">'
                            '<span class="_ds_collapse_pointer">▲</span>&nbsp;{}</a></th></tr>'
                            '<tr><td class="_n_details_content">{}</td></tr>'
                            '</table>').format(encoded_text, encoded_summary, decoded_summary, decoded_text)
        cursor.insertHtml(collapsible_html)

        # The table's (<td>) tag and adjacent text are converted into a pseudo block,
        # enclosed between \uFDD0 and \uFDD1:
        # print(f'{ord(b'\xef\xb7\x90'.decode('utf-8')):04X}')  # \uFDD0
        # print(f'{ord(b'\xef\xb7\x91'.decode('utf-8')):04X}')  # \uFDD1
        # # The entire block above is processed as a single object:
        # cursor.setPosition(start_pos, QTextCursor.MoveMode.MoveAnchor)
        # cursor.movePosition(QTextCursor.MoveOperation.NextCharacter, QTextCursor.MoveMode.KeepAnchor)

    def collapsible_click_event(self, url: QUrl, cursor: QTextCursor):
        # cursor.charFormat().anchorHref()
        self.logger.debug('Collapsible anchor clicked, scheme: %s, path: %s' % (url.scheme(), url.path()))

        cursor = self.process_anchor_label(cursor)

        encoded_text = url.path()
        self.logger.debug('Text to decode: {%s}', encoded_text)

        encoded_summary = url.fragment()
        decoded_summary = (base64.b64decode(encoded_summary.encode('utf-8'))
                           .decode('utf-8')
                           .strip())

        # Set default format for inserted text
        default_format = QTextCharFormat()
        cursor.setCharFormat(default_format)

        cursor.insertHtml('<p class="_ds_expand">{}'
                          '<a href="expandable:{}#{}" data-group="{}" data-level="{}">'
                          '<span class="_ds_expand_pointer">▼</span>&nbsp;{}</a>'
                          '</p>'
                          # Group and level aren't set here as the blocks have been already rendered
                          .format(self.zero_width_space, encoded_text, encoded_summary, 0, 0, decoded_summary))



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Color Picker dialog class.
- Functionality: Allows displaying, inserting, or copying a selected color.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import Qt, Signal
from PySide6.QtWidgets import QGridLayout, QDialog, QPushButton

from . import Settings
from . import Lexemes

from ..enums.colors import Colors

from functools import partial

import logging


class ColorPickerDialog(QDialog):

    color_selected = Signal(str)

    def __init__(self, parent=None):
        super().__init__(parent, Qt.WindowType.Window | Qt.WindowType.WindowStaysOnTopHint)

        self.parent = parent

        self.settings = Settings(parent=self)

        self.logger = logging.getLogger('color_picker_dialog')

        # Load lexemes for selected language and scope
        self.lexemes = Lexemes(self.settings.app_language, default_scope='color_picker_dialog')

        self.setWindowTitle(self.lexemes.get('color_picker_title'))
        self.setCursor(Qt.CursorShape.PointingHandCursor)

        self.layout = QGridLayout(self)
        self.layout.setSpacing(3)

        self.init_ui()

    def init_ui(self):
        row = col = 0
        for color in Colors:
            button = QPushButton(self)
            button.setToolTip(self.lexemes.get(f'color_picker_color_{color.name.lower()}'))
            # The style is not theme dependant, hence no need to move it out from the code.
            # The border color can be adjusted, place it to the theme's .ini file.
            button.setStyleSheet("""
            QPushButton {
                background-color: %s;
                border: 1px solid silver;
            }
            """ % color.name)
            button.setFixedSize(32, 32)
            button.clicked.connect(partial(self.color_chosen, color.name.lower()))
            self.layout.addWidget(button, row, col)
            col += 1
            if col >= 13:  # Based on colors count
                col = 0
                row += 1

    def color_chosen(self, color):
        self.color_selected.emit(color)
        # Close the dialog if any color selected
        self.accept()



# French lexemes toolbar.py
lexemes = {
    "actions_label_new_document": "Nouveau document",
    "actions_accessible_name_new_document": "Nouveau document",
    "actions_label_open": "Ouvrir",
    "actions_accessible_name_open": "Ouvrir le document",
    "actions_label_save": "Enregistrer",
    "actions_accessible_name_save": "Enregistrer le document",
    "actions_label_save_as": "Enregistrer sous",
    "actions_accessible_name_save_as": "Enregistrer une copie du document",
    "actions_label_sync": "Synchroniser",
    "actions_accessible_name_sync": "Synchroniser le document avec un stockage externe",
    "actions_label_edit": "Éditer",
    "actions_accessible_name_edit": "Éditer le fichier",
    "actions_label_source": "Source",
    "actions_accessible_name_source": "Code source",
    "actions_label_encrypt": "Chiffrer",
    "actions_accessible_name_encrypt": "Chiffrer le fichier",
    "actions_label_decrypt": "Déchiffrer",
    "actions_accessible_name_decrypt": "Déchiffrer le fichier",
    "actions_label_bold": "Gras",
    "actions_accessible_name_bold": "Format de texte en gras",
    "actions_label_italic": "Italique",
    "actions_accessible_name_italic": "Format de texte en italique",
    "actions_label_underline": "Souligner",
    "actions_accessible_name_underline": "Format de texte souligné",
    "actions_label_strikethrough": "Barré",
    "actions_accessible_name_strikethrough": "Format de texte barré",
    "actions_label_blockquote": "Citation",
    "actions_accessible_name_blockquote": "Format de texte en citation",
    "actions_label_color_picker": "Sélecteur de couleur",
    "actions_accessible_name_color_picker": "Choisir une couleur de texte",
    "actions_label_ai_assistant": "Assistant IA",
    "actions_accessible_name_ai_assistant": "Exécuter l'assistant IA",
    "actions_label_about": "À propos",
    "actions_accessible_name_about": "À propos de l'App",
    "actions_label_exit": "Quitter",
    "actions_accessible_name_exit": "Quitter l'App",
    "actions_label_settings": "Paramètres",
    "actions_accessible_name_settings": "Paramètres de l'application",

    "search_input_label": "Rechercher",
    "search_input_placeholder_text": "Rechercher du texte dans le document",
    "search_input_accessible_description": "Champ de recherche",
    "search_buttons_label_clear": "Effacer la recherche",
    "search_buttons_accessible_name_clear": "Effacer le champ de recherche",
    "search_buttons_label_prev": "Recherche précédente",
    "search_buttons_accessible_name_prev": "Résultat de recherche précédent",
    "search_buttons_label_next": "Recherche suivante",
    "search_buttons_accessible_name_next": "Résultat de recherche suivant",
    "search_case_sensitive_accessible_name": "Sensible à la casse",
    "search_case_sensitive_tooltip": "Recherche sensible à la casse",
}



# Portuguese lexemes common.py
lexemes = {
    "app_title": "Editor Notolog",
    "app_title_with_sub": "{app_title} - {sub_title}",

    "tree_filter_accessible_desc": "Campo de filtro de arquivo",

    "menu_action_copy_file_path": "Copiar caminho do arquivo",
    "menu_action_rename": "Renomear",
    "menu_action_delete": "Excluir",
    "menu_action_delete_completely": "Excluir completamente",
    "menu_action_restore": "Restaurar",
    "menu_action_create_new_dir": "Criar um novo diretório",

    "dialog_file_rename_title": "Renomear arquivo",
    "dialog_file_rename_field_label": "Digite o novo nome do arquivo",
    "dialog_file_rename_button_ok": "Renomear",
    "dialog_file_rename_warning_exists": "Já existe um arquivo com o mesmo nome",

    "dialog_file_delete_title": "Excluir arquivo",
    "dialog_file_delete_text": "Excluir arquivo \"{file_name}\"?",
    "dialog_file_delete_completely_title": "Excluir arquivo completamente",
    "dialog_file_delete_completely_text": "Excluir completamente o arquivo \"{file_name}\"?",
    "dialog_file_delete_error": "Não foi possível excluir o arquivo, ocorreu um erro",
    "dialog_file_delete_error_not_found": "Arquivo não encontrado",

    "dialog_file_restore_title": "Restaurar arquivo",
    "dialog_file_restore_text": "Restaurar o arquivo \"{file_name}\"?",
    "dialog_file_restore_error": "Não é possível restaurar o arquivo, ocorreu um erro",
    "dialog_file_restore_warning_exists": "Já existe um arquivo com o nome {file_name}",

    "dialog_create_new_dir_title": "Criar um novo diretório",
    "dialog_create_new_dir_label": "Nome do novo diretório",
    "dialog_create_new_dir_input_placeholder_text": "Digite o nome do diretório",
    "dialog_create_new_dir_button_ok": "Criar",
    "dialog_create_new_dir_button_cancel": "Cancelar",
    "dialog_create_new_dir_warning_empty_title": "Erro no nome do novo diretório",
    "dialog_create_new_dir_warning_empty_text": "O nome do diretório não pode estar vazio",
    "dialog_create_new_dir_warning_too_long_title": "Erro no nome do novo diretório",
    "dialog_create_new_dir_warning_too_long_text": "O nome do diretório é muito longo; máximo de "
                                                   "{symbols} caracteres permitidos!",
    "dialog_create_new_dir_error_existed": "O diretório já existe",
    "dialog_create_new_dir_error": "Não é possível criar o diretório. Certifique-se de que o diretório de destino "
                                   "{base_dir} seja gravável",

    "dialog_message_box_title": "Mensagem",
    "dialog_message_box_button_ok": "Fechar",

    "action_new_file_first_line_template_text": "Novo documento",
    "action_open_file_dialog_caption": "Abrir arquivo",
    "action_save_as_file_dialog_caption": "Salvar arquivo",

    "dialog_save_empty_file_title": "Salvar arquivo vazio",
    "dialog_save_empty_file_text": "Permitir salvar o arquivo com conteúdo vazio?",

    "dialog_encrypt_file_title": "Criptografar arquivo",
    "dialog_encrypt_file_text": "Criptografar arquivo \"{file_name}\"?",
    "encrypt_file_warning_file_is_already_encrypted": "O arquivo já está criptografado!",
    "dialog_encrypt_file_rewrite_existing_title": "Sobrescrever arquivo existente",
    "dialog_encrypt_file_rewrite_existing_text": "Sobrescrever arquivo existente \"{file_path}\"?",

    "dialog_decrypt_file_title": "Descriptografar arquivo",
    "dialog_decrypt_file_text": "Descriptografar arquivo \"{file_name}\"?",
    "decrypt_file_warning_file_is_not_encrypted": "O arquivo não está criptografado!",
    "dialog_decrypt_file_rewrite_existing_title": "Sobrescrever arquivo existente",
    "dialog_decrypt_file_rewrite_existing_text": "Sobrescrever arquivo existente \"{file_path}\"?",

    "dialog_encrypt_new_password_title": "Nova senha",
    "dialog_encrypt_new_password_label": "Senha:",
    "dialog_encrypt_new_password_input_placeholder_text": "Digite a nova senha",
    "dialog_encrypt_new_password_hint_label": "Dica:",
    "dialog_encrypt_new_password_hint_label_description":
        "A dica não é criptografada e pode ser lida do arquivo!"
        "\nNão use dicas óbvias que possam ser adivinhadas, como"
        "\na data de nascimento, ou algo do tipo. Tente usar uma referência.",
    "dialog_encrypt_new_password_hint_input_placeholder_text": "Digite uma dica (opcional)",
    "dialog_encrypt_new_password_button_ok": "OK",
    "dialog_encrypt_new_password_button_cancel": "Cancelar",
    "dialog_encrypt_new_password_warning_empty_title": "Aviso",
    "dialog_encrypt_new_password_warning_empty_text": "O campo de senha não pode ficar vazio!",
    "dialog_encrypt_new_password_warning_too_long_title": "Aviso",
    "dialog_encrypt_new_password_warning_too_long_text":
        "O campo de dica é muito longo, máximo de {symbols} caracteres!",

    "dialog_encrypt_password_title": "Digite a senha",
    "dialog_encrypt_password_label": "Senha:",
    "dialog_encrypt_password_input_placeholder_text": "Digite a senha",
    "dialog_encrypt_password_hint_label": "Dica:",
    "dialog_encrypt_password_button_ok": "OK",
    "dialog_encrypt_password_button_cancel": "Cancelar",

    "dialog_encrypt_password_reset_title": "Redefinir senha de criptografia",
    "dialog_encrypt_password_reset_text": "Você tem certeza que deseja redefinir a senha de criptografia atual?",
    "dialog_encrypt_password_reset_button_cancel": "Cancelar",
    "dialog_encrypt_password_reset_button_yes": "Sim",

    "dialog_open_link_title": "Link",
    "dialog_open_link_text": "Abrir link \"{url}\" no navegador?",

    "dialog_reset_settings_title": "Redefinir configurações?",
    "dialog_reset_settings_text":
        "Todos os dados armazenados nas configurações serão apagados, e o aplicativo será reiniciado para aplicar "
        "as mudanças.",

    "dialog_exit_unsaved_title": "Confirmar Saída",
    "dialog_exit_unsaved_text": "O arquivo aberto '{file_name}' não pode ser salvo. Continuar com a saída?",

    "message_app_config_file_access": "Permissão negada ao acessar o arquivo de configuração do aplicativo em {file_path}. "
                                      "Defina as permissões corretas para garantir a operação adequada.",

    "field_dir_path_dialog_caption": "Selecionar Diretório",
    "field_file_path_dialog_caption": "Selecionar Arquivo",

    "load_file_encryption_password_mismatch": "Senha de criptografia não confere!",
    "load_file_encryption_password_incorrect": "Senha de criptografia incorreta!",
    "load_file_none_content_error": "Não foi possível carregar o arquivo.",

    "action_new_file_error_occurred": "Não é possível criar o arquivo; ocorreu um erro."
                                      "\nVerifique as permissões do sistema de arquivos.",
    "save_active_file_error_occurred": "Não é possível salvar o arquivo; ocorreu um erro.",

    "expandable_block_default_title": "Mais informações...",
    "expandable_block_open_close_tags_mismatch_warning":
        "Incompatibilidade das tags de abertura/fechamento do bloco <details>",

    "dialog_color_picker_color_copied_to_the_clipboard": "O texto formatado foi copiado para a área de transferência",

    "popup_about_title": "Informações do Aplicativo",
    "popup_about_app_name_description": "Editor de Markdown Python",

    "popup_about_version": "Versão",
    "popup_about_license": "Licença",
    "popup_about_website": "Site",
    "popup_about_repository": "GitHub",
    "popup_about_pypi": "PyPi",
    "popup_about_date": "Data",

    "update_helper_new_version_is_available": "Nova versão {latest_version} do aplicativo disponível",
    "update_helper_latest_version_installed": "A versão mais recente do aplicativo está instalada",

    "network_connection_error_empty": "Não é possível obter informações de resposta",
    "network_connection_error_connection_or_dns":
        "Host não encontrado. Pode haver um problema com a conexão de internet ou DNS.",
    "network_connection_error_connection_refused":
        "Conexão recusada. O servidor pode estar inativo ou há problemas de rede.",
    "network_connection_error_connection_timed_out": "A conexão expirou. Pode haver problemas de rede.",
    "network_connection_error_connection_404_error":
        "Erro de conexão 404. A página ou recurso solicitado não foi encontrado.",
    "network_connection_error_generic_with_status_code": "Falha na solicitação com código de status: {status_code}",
}



# Lexica Communia Latine ai_assistant.py
lexemes = {
    # Assistens AI
    "dialog_title": "Postulatio ad AI",
    "dialog_prompt_input_placeholder_text": "Postulatio",
    "dialog_prompt_input_accessible_description": "Postulationem ingredi ad contextum assistenti AI praebendum",
    "dialog_response_output_placeholder_text": "Responsi data",
    "dialog_response_output_accessible_description": "Responsi data in hoc campo apparebunt",
    "dialog_response_output_notice_empty_text": "Praebe textum ad respondendum fere...",

    "dialog_usage_module_label": "MODULUS",
    "dialog_usage_model_label": "EXEMPLAR",
    "dialog_usage_tokens_label": "SIGNA",
    "dialog_usage_tokens_prompt": "promptus: {tokens}",
    "dialog_usage_tokens_answer": "responsio: {tokens}",
    "dialog_usage_tokens_total": "totum: {tokens}",

    "dialog_button_send_request": "Postulationem mittere",
    "dialog_button_stop_request": "Intermitte",
    "dialog_button_save_history": "Conferensiae historia serva",

    "dialog_message_copy_tooltip": "Exemplum textus",
    "dialog_message_copied_tooltip": "Exemplar",

    "dialog_error_loading_model": "Error loading mos proprius",
}



# Spanish lexemes toolbar.py
lexemes = {
    "actions_label_new_document": "Nuevo documento",
    "actions_accessible_name_new_document": "Nuevo documento",
    "actions_label_open": "Abrir",
    "actions_accessible_name_open": "Abrir documento",
    "actions_label_save": "Guardar",
    "actions_accessible_name_save": "Guardar documento",
    "actions_label_save_as": "Guardar como",
    "actions_accessible_name_save_as": "Guardar como copia del documento",
    "actions_label_sync": "Sincronizar",
    "actions_accessible_name_sync": "Sincronizar documento con un almacenamiento externo",
    "actions_label_edit": "Editar",
    "actions_accessible_name_edit": "Editar archivo",
    "actions_label_source": "Fuente",
    "actions_accessible_name_source": "Código fuente",
    "actions_label_encrypt": "Encriptar",
    "actions_accessible_name_encrypt": "Encriptar archivo",
    "actions_label_decrypt": "Desencriptar",
    "actions_accessible_name_decrypt": "Desencriptar archivo",
    "actions_label_bold": "Negrita",
    "actions_accessible_name_bold": "Formato de texto en negrita",
    "actions_label_italic": "Cursiva",
    "actions_accessible_name_italic": "Formato de texto en cursiva",
    "actions_label_underline": "Subrayar",
    "actions_accessible_name_underline": "Formato de texto subrayado",
    "actions_label_strikethrough": "Tachar",
    "actions_accessible_name_strikethrough": "Formato de texto tachado",
    "actions_label_blockquote": "Cita",
    "actions_accessible_name_blockquote": "Formato de texto en cita",
    "actions_label_color_picker": "Selector de color",
    "actions_accessible_name_color_picker": "Elegir un color de texto",
    "actions_label_ai_assistant": "Asistente IA",
    "actions_accessible_name_ai_assistant": "Ejecutar asistente IA",
    "actions_label_about": "Acerca de",
    "actions_accessible_name_about": "Acerca de la App",
    "actions_label_exit": "Salir",
    "actions_accessible_name_exit": "Salir de la App",
    "actions_label_settings": "Configuración",
    "actions_accessible_name_settings": "Configuración de la aplicación",

    "search_input_label": "Buscar",
    "search_input_placeholder_text": "Buscar texto en el documento",
    "search_input_accessible_description": "Campo de entrada de búsqueda",
    "search_buttons_label_clear": "Limpiar búsqueda",
    "search_buttons_accessible_name_clear": "Limpiar campo de búsqueda",
    "search_buttons_label_prev": "Búsqueda anterior",
    "search_buttons_accessible_name_prev": "Resultado de búsqueda anterior",
    "search_buttons_label_next": "Búsqueda siguiente",
    "search_buttons_accessible_name_next": "Resultado de búsqueda siguiente",
    "search_case_sensitive_accessible_name": "Sensible a mayúsculas",
    "search_case_sensitive_tooltip": "Búsqueda sensible a mayúsculas",
}



"""
Notolog Editor
An open-source Markdown editor built with Python.

File Details:
- Purpose: Provides app status bar UI.

Repository: https://github.com/notolog/notolog-editor
Website: https://notolog.app
PyPI: https://pypi.org/project/notolog

Author: Vadim Bakhrenkov
Copyright: 2024-2025 Vadim Bakhrenkov
License: MIT License

For detailed instructions and project information, please see the repository's README.md.
"""

from PySide6.QtCore import Qt
from PySide6.QtGui import QColor
from PySide6.QtWidgets import QStatusBar, QWidget, QLabel, QPushButton, QSizePolicy

import logging
from typing import TYPE_CHECKING

from . import Settings
from . import Lexemes
from . import ThemeHelper
from . import TooltipHelper

from .sort_filter_proxy_model import SortFilterProxyModel
from .vertical_line_spacer import VerticalLineSpacer

if TYPE_CHECKING:
    from typing import Union  # noqa


class StatusBar(QStatusBar):
    def __init__(self, parent=None):
        super().__init__(parent)

        self.parent = parent

        if self.parent and hasattr(self.parent, 'font'):
            # Apply font from the dialog instance to the label
            self.setFont(self.parent.font())

        self.logger = logging.getLogger('statusbar')

        self.settings = Settings(parent=self)

        self.theme_helper = ThemeHelper()

        # Load lexemes for selected language and scope
        self.lexemes = Lexemes(self.settings.app_language, default_scope='statusbar')

        self._elements = {}  # Label storage

        self.litter_bin_label = None  # type: Union[QPushButton, None]

        self.data_size_label = None  # type: Union[QLabel, None]
        self.mode_label = None  # type: Union[QLabel, None]
        self.save_progress_label = None  # type: Union[QLabel, None]
        self.encryption_label = None  # type: Union[QLabel, None]
        self.source_label = None  # type: Union[QLabel, None]
        self.cursor_label = None  # type: Union[QLabel, None]

        self.warning_label = None  # type: Union[QPushButton, None]

        self.init()

    def init(self):
        # Attach style to the QStatusBar and QLabel
        self.setStyleSheet(self.theme_helper.get_css('statusbar'))

        # Litter bin button to show/hide deleted files
        self.litter_bin_label = QPushButton(self)
        self.litter_bin_label.setCursor(Qt.CursorShape.PointingHandCursor)
        self.litter_bin_label.setFont(self.font())
        self.litter_bin_label.setObjectName('statusbar_litter_bin_label')
        self.litter_bin_label.setFlat(True)
        # Theme icon with a fallback to a system one if needed
        # self.theme_helper.get_color('statusbar_...')
        icon = self.theme_helper.get_icon(theme_icon='trash3.svg',
                                          color=QColor(self.theme_helper.get_color('statusbar_litter_bin_icon_color')))
        self.litter_bin_label.setIcon(icon)
        self.litter_bin_label.setText(self.lexemes.get('statusbar_litter_bin_label'))
        self.litter_bin_label.setToolTip(self.lexemes.get('statusbar_litter_bin_label'))
        self.litter_bin_label.setAccessibleName(self.lexemes.get('statusbar_litter_bin_accessible_name'))
        self.litter_bin_label.clicked.connect(lambda: self.toggle_litter_bin_button())

        self.addWidget(self.litter_bin_label)

        central_spacer = QWidget(self)
        central_spacer.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.addPermanentWidget(central_spacer)

        self.data_size_label = QLabel(self)
        self.data_size_label.setFont(self.font())
        self.addPermanentWidget(self.data_size_label)

        self.save_progress_label = QLabel(self)
        self.save_progress_label.setFont(self.font())
        self.save_progress_label.setVisible(False)
        self.save_progress_label.setText(self.lexemes.get('statusbar_save_progress_label'))
        self.addPermanentWidget(self.save_progress_label)

        self.addPermanentWidget(VerticalLineSpacer())

        self.mode_label = QLabel(self)
        self.mode_label.setFont(self.font())
        """
        Do not set object name similar to the lexeme to avoid double update with unprocessed lexeme's placeholders.
        E.g.: self.mode_label.setObjectName('statusbar_mode_label')
        """
        self.addPermanentWidget(self.mode_label)

        self.addPermanentWidget(VerticalLineSpacer())

        self.encryption_label = QLabel(self)
        self.encryption_label.setFont(self.font())
        self.addPermanentWidget(self.encryption_label)

        self.addPermanentWidget(VerticalLineSpacer())

        self.source_label = QLabel(self)
        self.source_label.setFont(self.font())
        self.addPermanentWidget(self.source_label)

        self.addPermanentWidget(VerticalLineSpacer())

        self.cursor_label = QLabel(self)
        self.cursor_label.setFont(self.font())
        self.addPermanentWidget(self.cursor_label)

        self.warning_label = QPushButton(self)
        self.warning_label.setVisible(False)
        self.warning_label.setFlat(True)
        self.warning_label.setFont(self.font())
        self.warning_label.setObjectName('statusbar_warning_label')
        icon = self.theme_helper.get_icon(theme_icon='exclamation-triangle-fill.svg',
                                          color=QColor(self.theme_helper.get_color('statusbar_warning_icon_color')))
        self.warning_label.setIcon(icon)
        self.addPermanentWidget(self.warning_label)
        self.warning_label.clicked.connect(
            lambda: TooltipHelper.show_tooltip(widget=self.warning_label, text=self.warning_label.toolTip()))

    def toggle_litter_bin_button(self):
        self.set_litter_bin_visibility(not self.settings.show_deleted_files)

    def set_litter_bin_visibility(self, visible: bool = False):
        if (hasattr(self, 'parent')
                and hasattr(self.parent, 'tree_proxy_model')
                and isinstance(self.parent.tree_proxy_model, SortFilterProxyModel)):
            # Get tree proxy model to obtain the data
            tree_proxy_model = self.parent.tree_proxy_model  # type: SortFilterProxyModel
            if visible:
                # Add 'deleted' extension to the tree
                tree_proxy_model.add_extension('del'),
                self.litter_bin_label.setIcon(self.theme_helper.get_icon(
                    theme_icon='trash3-fill.svg',
                    color=QColor(self.theme_helper.get_color('statusbar_litter_bin_icon_color_active'))))
                self.settings.show_deleted_files = True
            else:
                # Remove 'deleted' extension from the tree
                tree_proxy_model.remove_extension('del'),
                self.litter_bin_label.setIcon(self.theme_helper.get_icon(
                    theme_icon='trash3.svg',
                    color=QColor(self.theme_helper.get_color('statusbar_litter_bin_icon_color'))))
                self.settings.show_deleted_files = False

    def show_warning(self, visible: bool = False, tooltip: str = None):
        if visible:
            self.warning_label.setVisible(True)
            if tooltip:
                self.warning_label.setToolTip(tooltip)
        else:
            self.warning_label.setVisible(False)
            self.warning_label.setToolTip('')

    def __getitem__(self, name) -> QLabel:
        """
        To get items like:
            self.statusbar['mode_label']
        @param name: string name of the object
        @return: QLabel object
        """
        return self._elements.get(name, f"Label '{name}' not found")

    def __getattr__(self, name) -> QLabel:
        """
        To get items like:
            self.statusbar.mode_label
        @param name: string name of the object
        @return: QLabel object
        """
        return self._elements.get(name, f"Label '{name}' not found")

    def __setattr__(self, name, value):
        if name == '_attributes' or not name.endswith('_label'):
            super().__setattr__(name, value)
        else:
            if (isinstance(value, QLabel)
                    or isinstance(value, QPushButton)
                    or value is None):
                self._elements[name] = value
            else:
                self.logger.warning(f'Trying to set object that is not a QLabel type {type(value)}')



# Latin lexemes settings_dialog.py
lexemes = {
    "tab_ondevice_llm_config": "In Machina LLM",

    "module_ondevice_llm_config_label": "Modello LLM in Machina",
    "module_ondevice_llm_config_path_label": "Locatio Modeli ONNX",
    "module_ondevice_llm_config_path_input_placeholder_text": "Iter ad Directorium Modeli",
    "module_ondevice_llm_config_path_input_accessible_description":
        "Ager inductus cum selector ad viam directorii modeli, ubi ONNX files reperiuntur, designandum.\n"
        "Modeli fulciti sunt in formato ONNX, quod stat pro Open Neural Network Exchange, norma aperta\n"
        "formato pro machinalibus discendi modelis.",

    "module_ondevice_llm_config_response_temperature_label": "Temperatura: {temperature}",
    "module_ondevice_llm_config_response_temperature_input_accessible_description":
        "Casualitatem responsionum modeli accommodat. Valores altiores output variores producunt,\n"
        "dum valores inferiores responsiones praedicibiliores reddunt.",

    "module_ondevice_llm_config_response_max_tokens_label": "Maximi Responsionis Token",
    "module_ondevice_llm_config_response_max_tokens_input_accessible_description":
        "Numerum summum signorum in responsione recipiendis constituit, ut verba et punctuationes,\n"
        "longitudinem productionis moderans.",

    "module_ondevice_llm_config_prompt_history_size_label": "Magna Historia Prompti",
    "module_ondevice_llm_config_prompt_history_size_input_accessible_description":
        "Numerum inquisitionum in historia prompti, quam systema ad referentiam conservat, moderatur.\n"
        "Valor nihili inquisitiones infinitas permittit."
}



# Greek lexemes common.py
lexemes = {
    "app_title": "Επεξεργαστής Notolog",
    "app_title_with_sub": "{app_title} - {sub_title}",

    "tree_filter_accessible_desc": "Πεδίο φίλτρου αρχείου",

    "menu_action_copy_file_path": "Αντιγραφή διαδρομής αρχείου",
    "menu_action_rename": "Μετονομασία",
    "menu_action_delete": "Διαγραφή",
    "menu_action_delete_completely": "Ολοκληρωτική διαγραφή",
    "menu_action_restore": "Επαναφορά",
    "menu_action_create_new_dir": "Δημιουργία νέου καταλόγου",

    "dialog_file_rename_title": "Μετονομασία αρχείου",
    "dialog_file_rename_field_label": "Εισάγετε νέο όνομα αρχείου",
    "dialog_file_rename_button_ok": "Μετονομάστε",
    "dialog_file_rename_warning_exists": "Ένα αρχείο με το ίδιο όνομα υπάρχει ήδη",

    "dialog_file_delete_title": "Διαγραφή αρχείου",
    "dialog_file_delete_text": "Διαγραφή του αρχείου \"{file_name}\";",
    "dialog_file_delete_completely_title": "Ολοκληρωτική διαγραφή αρχείου",
    "dialog_file_delete_completely_text": "Ολοκληρωτική διαγραφή του αρχείου \"{file_name}\";",
    "dialog_file_delete_error": "Αδυναμία διαγραφής αρχείου, συνέβη σφάλμα",
    "dialog_file_delete_error_not_found": "Το αρχείο δεν βρέθηκε",

    "dialog_file_restore_title": "Επαναφορά αρχείου",
    "dialog_file_restore_text": "Επαναφορά του αρχείου \"{file_name}\";",
    "dialog_file_restore_error": "Αδυναμία επαναφοράς αρχείου, συνέβη σφάλμα",
    "dialog_file_restore_warning_exists": "Ένα αρχείο με το όνομα {file_name} υπάρχει ήδη",

    "dialog_create_new_dir_title": "Δημιουργία νέου καταλόγου",
    "dialog_create_new_dir_label": "Όνομα νέου καταλόγου",
    "dialog_create_new_dir_input_placeholder_text": "Εισαγάγετε το όνομα καταλόγου",
    "dialog_create_new_dir_button_ok": "Δημιουργία",
    "dialog_create_new_dir_button_cancel": "Ακύρωση",
    "dialog_create_new_dir_warning_empty_title": "Σφάλμα ονόματος νέου καταλόγου",
    "dialog_create_new_dir_warning_empty_text": "Το όνομα του καταλόγου δεν μπορεί να είναι κενό",
    "dialog_create_new_dir_warning_too_long_title": "Σφάλμα ονόματος νέου καταλόγου",
    "dialog_create_new_dir_warning_too_long_text": "Το όνομα του καταλόγου είναι πολύ μακρύ. Επιτρέπονται έως "
                                                   "{symbols} χαρακτήρες!",
    "dialog_create_new_dir_error_existed": "Ο κατάλογος υπάρχει ήδη",
    "dialog_create_new_dir_error": "Δεν είναι δυνατή η δημιουργία του καταλόγου. Βεβαιωθείτε ότι ο κατάλογος "
                                   "προορισμού {base_dir} είναι εγγράψιμος",

    "dialog_message_box_title": "Μήνυμα",
    "dialog_message_box_button_ok": "Κλείσιμο",

    "action_new_file_first_line_template_text": "Νέο έγγραφο",
    "action_open_file_dialog_caption": "Άνοιγμα Αρχείου",
    "action_save_as_file_dialog_caption": "Αποθήκευση Αρχείου",

    "dialog_save_empty_file_title": "Αποθήκευση κενού αρχείου",
    "dialog_save_empty_file_text": "Να επιτραπεί η αποθήκευση του αρχείου χωρίς περιεχόμενο;",

    "dialog_encrypt_file_title": "Κρυπτογράφηση αρχείου",
    "dialog_encrypt_file_text": "Κρυπτογράφηση του αρχείου \"{file_name}\";",
    "encrypt_file_warning_file_is_already_encrypted": "Το αρχείο είναι ήδη κρυπτογραφημένο!",
    "dialog_encrypt_file_rewrite_existing_title": "Επανεγγραφή υπάρχοντος αρχείου",
    "dialog_encrypt_file_rewrite_existing_text": "Επανεγγραφή υπάρχοντος αρχείου \"{file_path}\";",

    "dialog_decrypt_file_title": "Αποκρυπτογράφηση αρχείου",
    "dialog_decrypt_file_text": "Αποκρυπτογράφηση του αρχείου \"{file_name}\";",
    "decrypt_file_warning_file_is_not_encrypted": "Το αρχείο δεν είναι κρυπτογραφημένο!",
    "dialog_decrypt_file_rewrite_existing_title": "Επανεγγραφή υπάρχοντος αρχείου",
    "dialog_decrypt_file_rewrite_existing_text": "Επανεγγραφή υπάρχοντος αρχείου \"{file_path}\";",

    "dialog_encrypt_new_password_title": "Νέος Κωδικός",
    "dialog_encrypt_new_password_label": "Κωδικός:",
    "dialog_encrypt_new_password_input_placeholder_text": "Εισάγετε νέο κωδικό",
    "dialog_encrypt_new_password_hint_label": "Υπόδειξη:",
    "dialog_encrypt_new_password_hint_label_description":
        "Η υπόδειξη δεν είναι κρυπτογραφημένη και μπορεί να διαβαστεί "
        "από το αρχείο!\nΑποφύγετε προφανείς υποδείξεις που μπορεί να "
        "μαντευτούν εύκολα, όπως ημερομηνίες γέννησης.\nΠροσπαθήστε "
        "να χρησιμοποιήσετε μια αναφορά που δεν συνδέεται εύκολα με εσάς.",
    "dialog_encrypt_new_password_hint_input_placeholder_text": "Εισάγετε υπόδειξη (προαιρετικά)",
    "dialog_encrypt_new_password_button_ok": "Εντάξει",
    "dialog_encrypt_new_password_button_cancel": "Ακύρωση",
    "dialog_encrypt_new_password_warning_empty_title": "Προειδοποίηση",
    "dialog_encrypt_new_password_warning_empty_text": "Το πεδίο του κωδικού δεν μπορεί να είναι άδειο!",
    "dialog_encrypt_new_password_warning_too_long_title": "Προειδοποίηση",
    "dialog_encrypt_new_password_warning_too_long_text":
        "Το πεδίο της υπόδειξης είναι πολύ μακρύ, μέγιστο {symbols} χαρακτήρες!",

    "dialog_encrypt_password_title": "Εισαγωγή Κωδικού",
    "dialog_encrypt_password_label": "Κωδικός:",
    "dialog_encrypt_password_input_placeholder_text": "Εισάγετε κωδικό",
    "dialog_encrypt_password_hint_label": "Υπόδειξη:",
    "dialog_encrypt_password_button_ok": "Εντάξει",
    "dialog_encrypt_password_button_cancel": "Ακύρωση",

    "dialog_encrypt_password_reset_title": "Επαναφορά Κωδικού Κρυπτογράφησης",
    "dialog_encrypt_password_reset_text":
        "Είστε σίγουροι ότι θέλετε να επαναφέρετε τον τρέχοντα κωδικό κρυπτογράφησης;",
    "dialog_encrypt_password_reset_button_cancel": "Ακύρωση",
    "dialog_encrypt_password_reset_button_yes": "Ναι",

    "dialog_open_link_title": "Σύνδεσμος",
    "dialog_open_link_text": "Άνοιγμα συνδέσμου \"{url}\" σε πρόγραμμα περιήγησης;",

    "dialog_reset_settings_title": "Επαναφορά ρυθμίσεων;",
    "dialog_reset_settings_text":
        "Όλα τα αποθηκευμένα δεδομένα στις ρυθμίσεις θα διαγραφούν, και η εφαρμογή θα επανεκκινηθεί για να εφαρμοστούν "
        "οι αλλαγές.",

    "dialog_exit_unsaved_title": "Επιβεβαίωση Εξόδου",
    "dialog_exit_unsaved_text": "Το ανοιχτό αρχείο '{file_name}' δεν μπορεί να αποθηκευτεί. Να συνεχίσουμε με την έξοδο;",

    "message_app_config_file_access": "Άρνηση πρόσβασης κατά την πρόσβαση στο αρχείο διαμόρφωσης εφαρμογής στο {file_path}. "
                                      "Ορίστε τα σωστά δικαιώματα για να διασφαλίσετε τη σωστή λειτουργία.",

    "field_dir_path_dialog_caption": "Επιλέξτε Κατάλογο",
    "field_file_path_dialog_caption": "Επιλέξτε Αρχείο",

    "load_file_encryption_password_mismatch": "Αναντιστοιχία κωδικού κρυπτογράφησης!",
    "load_file_encryption_password_incorrect": "Λάθος κωδικός κρυπτογράφησης!",
    "load_file_none_content_error": "Το αρχείο δεν μπορεί να φορτωθεί.",

    "action_new_file_error_occurred": "Δεν είναι δυνατή η δημιουργία αρχείου· προέκυψε σφάλμα."
                                      "\nΕλέγξτε τα δικαιώματα του συστήματος αρχείων.",
    "save_active_file_error_occurred": "Δεν είναι δυνατή η αποθήκευση αρχείου· προέκυψε σφάλμα.",

    "expandable_block_default_title": "Περισσότερες πληροφορίες...",
    "expandable_block_open_close_tags_mismatch_warning": "Αντιφατική ανοιχτή/κλειστή ετικέτα <details> μπλοκ",

    "dialog_color_picker_color_copied_to_the_clipboard": "Το μορφοποιημένο κείμενο έχει αντιγραφεί στο πρόχειρο",

    "popup_about_title": "Πληροφορίες Εφαρμογής",
    "popup_about_app_name_description": "Επεξεργαστής Markdown της Python",

    "popup_about_version": "Έκδοση",
    "popup_about_license": "Άδεια",
    "popup_about_website": "Ιστοσελίδα",
    "popup_about_repository": "GitHub",
    "popup_about_pypi": "PyPi",
    "popup_about_date": "Ημερομηνία",

    "update_helper_new_version_is_available": "Μια νέα έκδοση {latest_version} της εφαρμογής είναι διαθέσιμη",
    "update_helper_latest_version_installed": "Η τελευταία έκδοση της εφαρμογής είναι εγκατεστημένη",

    "network_connection_error_empty": "Δεν είναι δυνατή η λήψη πληροφοριών απόκρισης",
    "network_connection_error_connection_or_dns":
        "Ο ξενιστής δεν βρέθηκε. Μπορεί να υπάρχει ένα ζήτημα με τη σύνδεση στο διαδίκτυο ή DNS.",
    "network_connection_error_connection_refused":
        "Η σύνδεση απορρίφθηκε. Μπορεί ο διακομιστής να είναι κάτω, ή μπορεί να υπάρχουν ζητήματα δικτύου.",
    "network_connection_error_connection_timed_out": "Η σύνδεση έληξε. Μπορεί να υπάρχουν ζητήματα δικτύου.",
    "network_connection_error_connection_404_error":
        "Σφάλμα 404 σύνδεσης. Η ζητούμενη σελίδα ή πόρος δεν βρέθηκε.",
    "network_connection_error_generic_with_status_code": "Η αίτηση απέτυχε με κωδικό κατάστασης: {status_code}",
}



# Finnish lexemes common.py
lexemes = {
    "module_llama_cpp_name": "Moduuli llama.cpp",

    "module_llama_cpp_model_exception": "Mallia ei löydy virhe: {error_msg}",
    "module_llama_cpp_task_exception": "Malligeneraattoria ei voi alustaa: {error_msg}",
}



# Hindi lexemes color_picker_dialog.py
lexemes = {
    "color_picker_title": "रंग चुनें",

    "color_picker_color_black": "काला",
    "color_picker_color_silver": "चांदी",
    "color_picker_color_gray": "धूसर",
    "color_picker_color_white": "सफेद",
    "color_picker_color_maroon": "मैरून",
    "color_picker_color_red": "लाल",
    "color_picker_color_purple": "बैंगनी",
    "color_picker_color_fuchsia": "फ्यूशिया",
    "color_picker_color_green": "हरा",
    "color_picker_color_lime": "चूना",
    "color_picker_color_olive": "जैतूनी",
    "color_picker_color_yellow": "पीला",
    "color_picker_color_navy": "नेवी ब्लू",
    "color_picker_color_blue": "नीला",
    "color_picker_color_teal": "टील",
    "color_picker_color_aqua": "ऐक्वा",
    "color_picker_color_aliceblue": "ऐलिस ब्लू",
    "color_picker_color_antiquewhite": "एंटीक सफेद",
    "color_picker_color_aquamarine": "एक्वामरीन",
    "color_picker_color_azure": "आसमानी",
    "color_picker_color_beige": "बेज",
    "color_picker_color_bisque": "बिस्क",
    "color_picker_color_blanchedalmond": "ब्लांच्ड बादाम",
    "color_picker_color_blueviolet": "नीली बैंगनी",
    "color_picker_color_brown": "भूरा",
    "color_picker_color_burlywood": "बर्लीवुड",
    "color_picker_color_cadetblue": "कैडेट ब्लू",
    "color_picker_color_chartreuse": "शार्ट्र्यूज़",
    "color_picker_color_chocolate": "चॉकलेट",
    "color_picker_color_coral": "कोरल",
    "color_picker_color_cornflowerblue": "कॉर्नफ्लावर ब्लू",
    "color_picker_color_cornsilk": "कॉर्नसिल्क",
    "color_picker_color_crimson": "क्रिमसन",
    "color_picker_color_cyan": "सियान",
    "color_picker_color_darkblue": "गहरा नीला",
    "color_picker_color_darkcyan": "गहरा सियान",
    "color_picker_color_darkgoldenrod": "गहरा गोल्डन रॉड",
    "color_picker_color_darkgray": "गहरा धूसर",
    "color_picker_color_darkgreen": "गहरा हरा",
    "color_picker_color_darkkhaki": "गहरा खाकी",
    "color_picker_color_darkmagenta": "गहरा मैजेंटा",
    "color_picker_color_darkolivegreen": "गहरा जैतूनी हरा",
    "color_picker_color_darkorange": "गहरा नारंगी",
    "color_picker_color_darkorchid": "गहरा ऑर्किड",
    "color_picker_color_darkred": "गहरा लाल",
    "color_picker_color_darksalmon": "गहरा साल्मन",
    "color_picker_color_darkseagreen": "गहरा समुद्री हरा",
    "color_picker_color_darkslateblue": "गहरा स्लेट नीला",
    "color_picker_color_darkslategray": "गहरा स्लेट धूसर",
    "color_picker_color_darkturquoise": "गहरा फ़िरोज़ा",
    "color_picker_color_darkviolet": "गहरी बैंगनी",
    "color_picker_color_deeppink": "गहरा गुलाबी",
    "color_picker_color_deepskyblue": "गहरा आसमानी नीला",
    "color_picker_color_dimgray": "मद्धम धूसर",
    "color_picker_color_dodgerblue": "डॉजर ब्लू",
    "color_picker_color_firebrick": "फ़ायर ब्रिक",
    "color_picker_color_floralwhite": "फ़्लोरल सफेद",
    "color_picker_color_forestgreen": "वन हरा",
    "color_picker_color_gainsboro": "गेन्सबोरो",
    "color_picker_color_ghostwhite": "घोस्ट सफेद",
    "color_picker_color_gold": "सोना",
    "color_picker_color_goldenrod": "गोल्डन रॉड",
    "color_picker_color_greenyellow": "हरा पीला",
    "color_picker_color_honeydew": "हनी ड्यू",
    "color_picker_color_hotpink": "हॉट पिंक",
    "color_picker_color_indianred": "इंडियन लाल",
    "color_picker_color_indigo": "इंडिगो",
    "color_picker_color_ivory": "हाथीदांत",
    "color_picker_color_khaki": "खाकी",
    "color_picker_color_lavender": "लैवेंडर",
    "color_picker_color_lavenderblush": "लैवेंडर ब्लश",
    "color_picker_color_lawngreen": "लॉन हरा",
    "color_picker_color_lemonchiffon": "लेमन शिफॉन",
    "color_picker_color_lightblue": "हल्का नीला",
    "color_picker_color_lightcoral": "हल्का कोरल",
    "color_picker_color_lightcyan": "हल्का सियान",
    "color_picker_color_lightgoldenrodyellow": "हल्का गोल्डन रॉड पीला",
    "color_picker_color_lightgray": "हल्का धूसर",
    "color_picker_color_lightgreen": "हल्का हरा",
    "color_picker_color_lightpink": "हल्का गुलाबी",
    "color_picker_color_lightsalmon": "हल्का साल्मन",
    "color_picker_color_lightseagreen": "हल्का समुद्री हरा",
    "color_picker_color_lightskyblue": "हल्का आसमानी नीला",
    "color_picker_color_lightslategray": "हल्का स्लेट धूसर",
    "color_picker_color_lightsteelblue": "हल्का स्टील नीला",
    "color_picker_color_lightyellow": "हल्का पीला",
    "color_picker_color_limegreen": "लाइम हरा",
    "color_picker_color_linen": "लिनेन",
    "color_picker_color_magenta": "मैजेंटा",
    "color_picker_color_mediumaquamarine": "मध्यम एक्वामरीन",
    "color_picker_color_mediumblue": "मध्यम नीला",
    "color_picker_color_mediumorchid": "मध्यम ऑर्किड",
    "color_picker_color_mediumpurple": "मध्यम बैंगनी",
    "color_picker_color_mediumseagreen": "मध्यम समुद्री हरा",
    "color_picker_color_mediumslateblue": "मध्यम स्लेट नीला",
    "color_picker_color_mediumspringgreen": "मध्यम स्प्रिंग हरा",
    "color_picker_color_mediumturquoise": "मध्यम फ़िरोज़ा",
    "color_picker_color_mediumvioletred": "मध्यम वायलेट लाल",
    "color_picker_color_midnightblue": "मिडनाइट नीला",
    "color_picker_color_mintcream": "मिंट क्रीम",
    "color_picker_color_mistyrose": "मिस्टी रोज़",
    "color_picker_color_moccasin": "मोकासिन",
    "color_picker_color_navajowhite": "नवाजो सफेद",
    "color_picker_color_oldlace": "ओल्ड लेस",
    "color_picker_color_olivedrab": "ऑलिव ड्रैब",
    "color_picker_color_orange": "नारंगी",
    "color_picker_color_orangered": "ऑरेंज रेड",
    "color_picker_color_orchid": "ऑर्किड",
    "color_picker_color_palegoldenrod": "फीका गोल्डन रॉड",
    "color_picker_color_palegreen": "फीका हरा",
    "color_picker_color_paleturquoise": "फीका फ़िरोज़ा",
    "color_picker_color_palevioletred": "फीका वायलेट लाल",
    "color_picker_color_papayawhip": "पपाया व्हिप",
    "color_picker_color_peachpuff": "पीच पफ",
    "color_picker_color_peru": "पेरू",
    "color_picker_color_pink": "गुलाबी",
    "color_picker_color_plum": "प्लम",
    "color_picker_color_powderblue": "पाउडर नीला",
    "color_picker_color_rebeccapurple": "रिबेका पर्पल",
    "color_picker_color_rosybrown": "रोज़ी भूरा",
    "color_picker_color_royalblue": "रॉयल नीला",
    "color_picker_color_saddlebrown": "सैडल भूरा",
    "color_picker_color_salmon": "साल्मन",
    "color_picker_color_sandybrown": "सैंडी भूरा",
    "color_picker_color_seagreen": "समुद्री हरा",
    "color_picker_color_seashell": "सी शेल",
    "color_picker_color_sienna": "सिएना",
    "color_picker_color_skyblue": "आसमानी नीला",
    "color_picker_color_slateblue": "स्लेट नीला",
    "color_picker_color_slategray": "स्लेट धूसर",
    "color_picker_color_snow": "स्नो",
    "color_picker_color_springgreen": "स्प्रिंग हरा",
    "color_picker_color_steelblue": "स्टील नीला",
    "color_picker_color_tan": "टैन",
    "color_picker_color_thistle": "थिसल",
    "color_picker_color_tomato": "टमाटर",
    "color_picker_color_turquoise": "फ़िरोज़ा",
    "color_picker_color_violet": "बैंगनी",
    "color_picker_color_wheat": "गेहूँ",
    "color_picker_color_whitesmoke": "सफेद धुआँ",
    "color_picker_color_yellowgreen": "पीला हरा"
}
